#!/usr/bin/python
#dirsize.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#

import os
import sys

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in dirsize.py')

from twisted.internet import threads

import dhnio
import diskspace

_Jobs = {}
_Dirs = {}

def ask(dirpath, callback=None, arg=None):
    global _Jobs
    global _Dirs
    dhnio.Dprint(6, 'dirsize.ask %s' % dirpath)
    if _Jobs.has_key(dirpath):
        return 'counting size'
    if not os.path.isdir(dirpath):
        _Dirs[dirpath] = 'not exist'
        if callback:
            reactor.callLater(0, callback, 'not exist', arg)
        return 'not exist'
    d = threads.deferToThread(dhnio.getDirectorySize, dirpath)
    d.addCallback(done, dirpath)
    _Jobs[dirpath] = (d, callback, arg)
    _Dirs[dirpath] = 'counting size'
    return 'counting size'
    
def done(size, dirpath):
    global _Dirs
    global _Jobs
    dhnio.Dprint(6, 'dirsize.done %s %s' % (str(size), dirpath.decode(),))
    _Dirs[dirpath] = str(size)
    try:
        (d, cb, arg) = _Jobs.pop(dirpath, (None, None, None))
        if cb:
            cb(dirpath, size, arg)
    except:
        dhnio.DprintException()
    
def get(dirpath, default=''):
    global _Dirs
    return _Dirs.get(dirpath, default)
    
def isjob(dirpath):
    global _Jobs
    return _Jobs.has_key(dirpath)

def getLabel(dirpath):
    global _Dirs
    s = _Dirs.get(dirpath, '')
    if s not in ['counting size', 'not exist']:
        try:
            return diskspace.MakeStringFromBytes(int(s))
        except:
            return str(s)
    return str(s)
    
def getInBytes(dirpath):
    return diskspace.GetBytesFromString(get(dirpath))     
        
    


#!/usr/bin/python
#block_rebuilder.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#

import os
import sys
import time
import random


try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in block_rebuilder.py')

from twisted.internet.defer import Deferred, maybeDeferred
from twisted.internet import threads


import lib.dhnio as dhnio
import lib.misc as misc
import lib.nameurl as nameurl
import lib.transport_control as transport_control
import lib.settings as settings
import lib.contacts as contacts
import lib.eccmap as eccmap
import lib.tmpfile as tmpfile
import lib.diskspace as diskspace
from lib.automat import Automat


import fire_hire
import backup_rebuilder
import list_files_orator 
import lib.automats as automats

import backups
import raidread
import customerservice
import io_throttle
import backup_db
import contacts_status

_BlockRebuilder = None

#------------------------------------------------------------------------------ 

def A(event=None, arg=None):
    global _BlockRebuilder
    if _BlockRebuilder is None:
        _BlockRebuilder = BlockRebuilder()
    if event is not None:
        _BlockRebuilder.automat(event, arg)
    return _BlockRebuilder


class BlockRebuilder(Automat):
    timers = {'timer-1min':     (60,    ['REQUEST',]),
              'timer-1sec':     (1,     ['REQUEST', 'SENDING']),}
    
    def __init__(self,  
                 eccMap, 
                 backupID, 
                 blockNum, 
                 supplierSet, 
                 remoteData, 
                 remoteParity,
                 localData, 
                 localParity, 
                 creatorId = None, 
                 ownerId = None):
        self.eccMap = eccMap
        self.backupID = backupID
        self.blockNum = blockNum
        self.supplierSet = supplierSet
        self.supplierCount = len(self.supplierSet.suppliers)
        self.remoteData = remoteData
        self.remoteParity = remoteParity
        self.localData = localData
        self.localParity = localParity
        self.creatorId = creatorId
        self.ownerId = ownerId
        # at some point we may be dealing with when we're scrubbers
        if self.creatorId == None:
            self.creatorId = misc.getLocalID()
        if self.ownerId == None:
            self.ownerId = misc.getLocalID()
        # this files we want to rebuild
        # need to identify which files to work on
        self.missingData = [0] * self.supplierCount
        self.missingParity = [0] * self.supplierCount
        # list of packets ID we requested
        self.requestedFilesList = []
        # list of files to be send
        self.outstandingFilesList = []
        # time when we start sending files
        self.sendingStartedTime = 0
        # timeout to send a single file to supplier,
        # *2 because we want to be sure that it is not handled by io_throttle - it counts timeout too
        self.timeoutSending = 2 * int(settings.PacketSizeTarget() / settings.SendingSpeedLimit())
        # 1 if we sent a Data file to single supplier, 2 if get Ack from him, 3 if it was failed   
        self.dataSent = [0] * self.supplierCount
        # same for Parity files
        self.paritySent = [0] * self.supplierCount
        # for every packet we sending - remember if it was success or failed  
        self.sentReports = {}
        # dhnio.Dprint(8, 'block_rebuilder._BlockRebuilder blockNum=%s remote:{%s %s} local:{%s %s}' % (self.blockNum, self.remoteData, self.remoteParity, self.localData, self.localParity))
        Automat.__init__(self, 'block_rebuilder', 'AT_STARTUP', 10)
        
    def state_changed(self, oldstate, newstate):
        backup_rebuilder.A('block_rebuilder.state', newstate)
    
    def A(self, event, arg):
        #---AT_STARTUP---
        if self.state == 'AT_STARTUP':
            if event == 'start':
                self.doIdentifyMissingFiles()
                self.state = 'MISSING'
        #---MISSING---
        elif self.state == 'MISSING':
            if event == 'missing-files-identified' and not self.isMissingFilesOnHand():
                self.doRequestMissingFiles()
                self.state = 'REQUEST'
            elif event == 'missing-files-identified' and self.isMissingFilesOnHand():
                self.doRebuild()
                self.state = 'REBUILDING'
        #---REQUEST---
        elif self.state == 'REQUEST':
            if not self.isStopped() and ( ( event == 'timer-1sec' and self.isAllFilesReceived() ) or event == 'timer-1min' ):
                self.doRebuild()
                self.state = 'REBUILDING'
            elif event == 'timer-1sec' and self.isStopped():
                self.doDestroyMe()
                self.state = 'STOPPED'
        #---REBUILDING---
        elif self.state == 'REBUILDING':
            if event == 'rebuilding-finished' and self.isOutstandingFiles():
                self.doSendOutstandingFiles()
                self.state = 'SENDING'
            elif event == 'rebuilding-finished' and not self.isOutstandingFiles():
                self.doDestroyMe()
                self.state = 'FINISHED'
        #--SENDING---
        elif self.state == 'SENDING':
            if event == 'file-sent-report' and self.isAllFilesSent():
                self.doRemoveBlockFiles()
                self.doWorkDoneReport()
                self.doDestroyMe()
                self.state = 'DONE'
            elif event == 'timer-1sec' and self.isStopped():
                self.doWorkDoneReport()
                self.doDestroyMe()
                self.state = 'STOPPED'
            elif event == 'timer-1sec' and self.isTimeoutSending():
                self.doUpdateSentReports()
                self.doWorkDoneReport()
                self.doDestroyMe()
                self.state = 'TIMEOUT'
        #---FINISHED---
        elif self.state == 'FINISHED':
            pass
        #---STOPPED---
        elif self.state == 'STOPPED':
            pass
        #---DONE---
        elif self.state == 'DONE':
            pass
        #---TIMEOUT---
        elif self.state == 'TIMEOUT':
            pass
        
        
    def isStopped(self):
        return backup_rebuilder.ReadStoppedFlag()
    
    def isMissingFilesOnHand(self):
        for supplierNum in range(self.supplierCount):
            # if supplier do not have the Data but is on line 
            if self.missingData[supplierNum] == 1:
                # ... and we also do not have the Data 
                if self.localData[supplierNum] != 1:
                    # return False - will need request the file   
                    return False
            # same for Parity                
            if self.missingParity[supplierNum] == 1:
                if self.localParity[supplierNum] != 1:
                    return False
        #dhnio.Dprint(8, 'block_rebuilder.isMissingFilesOnHand return True')
        return True
        
    def isAllFilesReceived(self):
        return len(self.requestedFilesList) == 0
    
    def isAllFilesSent(self):
        return ( 1 not in self.dataSent ) and ( 1 not in self.paritySent )
    
    def isOutstandingFiles(self):
        return len(self.outstandingFilesList) > 0
    
    def isTimeoutSending(self):
        result = time.time() - self.sendingStartedTime > self.timeoutSending
        if result:
            dhnio.Dprint(6, 'block_rebuilder.isTimeoutSending return True, timeout=%d' % self.timeoutSending)
        return result
    
    def doIdentifyMissingFiles(self):
        def do_identify():
            self.availableSuppliers = self.supplierSet.GetActiveArray()
            for supplierNum in range(self.supplierCount):
                if self.availableSuppliers[supplierNum] == 0:
                    continue
                # if remote Data file not exist and supplier is online
                # we mark it as missing and will try to rebuild this file and send to him
                if self.remoteData[supplierNum] != 1:
                    # mark file as missing  
                    self.missingData[supplierNum] = 1
                # same for Parity file
                if self.remoteParity[supplierNum] != 1:
                    self.missingParity[supplierNum] = 1
            return True
        maybeDeferred(do_identify).addCallback(
            lambda x: self.automat('missing-files-identified'))
        
    def doRequestMissingFiles(self):
        self.availableSuppliers = self.supplierSet.GetActiveArray()
        # at the moment I'm download
        # everything I have available and needed
        for supplierNum in range(self.supplierCount):
            # if supplier is not alive - we can't request from him           
            if self.availableSuppliers[supplierNum] == 0:
                continue
            supplierID = self.supplierSet.suppliers[supplierNum]      
            # if the remote Data exist and is available because supplier is on line,
            # but we do not have it on hand - do request  
            if self.remoteData[supplierNum] == 1 and self.localData[supplierNum] == 0:
                PacketID = self.BuildFileName(supplierNum, 'Data')
                io_throttle.QueueRequestFile(self.FileReceived, self.creatorId, PacketID, self.ownerId, supplierID)
                self.requestedFilesList.append(PacketID)
            # same for Parity
            if self.remoteParity[supplierNum] == 1 and self.localParity[supplierNum] == 0:
                PacketID = self.BuildFileName(supplierNum, 'Parity')
                io_throttle.QueueRequestFile(self.FileReceived, self.creatorId, PacketID, self.ownerId, supplierID)
                self.requestedFilesList.append(PacketID)
        
    def doRebuild(self):
#        threads.deferToThread(self.AttemptRepair).addCallback(
#            lambda x: self.automat('rebuilding-thread-finished'))
        maybeDeferred(self.AttemptRebuild).addCallback(
            lambda x: self.automat('rebuilding-finished'))
        
    def doSendOutstandingFiles(self):
        for FileName, PacketID, supplierNum in self.outstandingFilesList:
            self.SendFile(FileName, PacketID, supplierNum)
        del self.outstandingFilesList
        self.outstandingFilesList = [] 
        self.sendingStartedTime = time.time()

    def doDestroyMe(self):
        automats.get_automats_by_index().pop(self.index)
        reactor.callLater(0, backup_rebuilder.RemoveBlockRebuilder, self)
    
    def doRemoveBlockFiles(self):
        # we want to remove files for this block 
        # because we only need them during rebuilding
        if settings.getGeneralLocalBackups() is True:
            # if user set this in settings - he want to keep the local files
            return
        # ... user do not want to keep local backups
        if settings.getGeneralWaitSuppliers() is True:
            # but he want to be sure - all suppliers are green for long time
            if contacts_status.hasOfflineSuppliers() or time.time() - fire_hire.GetLastFireTime() < 24*60*60:
                # some people are not there or we do not have stable team yet.
                # do not remove the files because we need it to rebuild
                return
        dhnio.Dprint(2, 'block_rebuilder.doRemoveBlockFiles [%s] [%s]' % (self.remoteData, self.remoteParity))
        for supplierNum in range(self.supplierCount):
            # data_filename = os.path.join(tmpfile.subdir('data-par'), self.BuildFileName(supplierNum, 'Data'))
            # parity_filename = os.path.join(tmpfile.subdir('data-par'), self.BuildFileName(supplierNum, 'Parity'))
            data_filename = os.path.join(settings.getLocalBackupsDir(), self.BuildFileName(supplierNum, 'Data'))
            parity_filename = os.path.join(settings.getLocalBackupsDir(), self.BuildFileName(supplierNum, 'Parity'))
            if os.path.isfile(data_filename) and self.remoteData[supplierNum] == 1:
                try:
                    os.remove(data_filename)
                    #dhnio.Dprint(6, '    ' + os.path.basename(data_filename))
                except:
                    dhnio.DprintException()
            if os.path.isfile(parity_filename) and self.remoteParity[supplierNum] == 1:
                try:
                    os.remove(parity_filename)
                    #dhnio.Dprint(6, '    ' + os.path.basename(parity_filename))
                except:
                    dhnio.DprintException()
        
    def doWorkDoneReport(self):
        if backup_rebuilder.ReadStoppedFlag() is False:
            backups.RebuildReport(self.backupID, self.blockNum, self.remoteData, self.remoteParity, self.sentReports)

    def doUpdateSentReports(self):
        for supplierNum in range(self.supplierCount):
            if self.dataSent[supplierNum] == 1:
                packetID = self.BuildFileName(supplierNum, 'Data')
                self.sentReports[packetID] = 'timeout2' 
                dhnio.Dprint(6, 'block_rebuilder.doUpdateSentReports %s' % packetID)
            if self.paritySent[supplierNum] == 1:
                packetID = self.BuildFileName(supplierNum, 'Parity')
                self.sentReports[packetID] = 'timeout2' 
                dhnio.Dprint(6, 'block_rebuilder.doUpdateSentReports %s' % packetID)
             
    #------------------------------------------------------------------------------ 

    def HaveAllData(self, parityMap):
        for segment in parityMap:
            if self.localData[segment] == 0:
                return False
        return True


    def AttemptRebuild(self):
        #dhnio.Dprint(6, 'block_rebuilder.AttemptRepair BEGIN')
        madeProgress = True
        while madeProgress:
            madeProgress = False
            # will check all data packets we have 
            for supplierNum in range(self.supplierCount):
                dataFileName = self.BuildRaidFileName(supplierNum, 'Data')
                # if we do not have this item on hands - we will reconstruct it from other items 
                if self.localData[supplierNum] == 0:
                    parityNum, parityMap = self.eccMap.GetDataFixPath(self.localData, self.localParity, supplierNum)
                    if parityNum != -1:
                        rebuildFileList = []
                        rebuildFileList.append(self.BuildRaidFileName(parityNum, 'Parity'))
                        for supplierParity in parityMap:
                            if supplierParity != supplierNum:
                                rebuildFileList.append(self.BuildRaidFileName(supplierParity, 'Data'))
                        raidread.RebuildOne(rebuildFileList, len(rebuildFileList), dataFileName)
                    #send the rebuilt file back, set the missing to zero
                    if os.path.exists(dataFileName):
                        self.localData[supplierNum] = 1
                        madeProgress = True
                        dhnio.Dprint(8, 'block_rebuilder.AttemptRebuild made progress with supplier %d' % supplierNum)
                # now we check again if we have the data on hand after rebuild at it is missing - send it
                # but also check to not duplicate sending to this man            
                if self.localData[supplierNum] == 1 and self.missingData[supplierNum] == 1 and self.dataSent[supplierNum] == 0:
                    dhnio.Dprint(8, "block_rebuilder.AttemptRebuild new outstanding Data for supplier %d" % supplierNum)
                    self.outstandingFilesList.append((dataFileName, self.BuildFileName(supplierNum, 'Data'), supplierNum))
                    self.dataSent[supplierNum] = 1
        # now with parities ...            
        for supplierNum in range(self.supplierCount):
            parityFileName = self.BuildRaidFileName(supplierNum, 'Parity')
            if self.localParity[supplierNum] == 0:
                parityMap = self.eccMap.ParityToData[supplierNum]
                if self.HaveAllData(parityMap):
                    rebuildFileList = []
                    for supplierParity in parityMap:
                        rebuildFileList.append(self.BuildRaidFileName(supplierParity, 'Data'))
                    raidread.RebuildOne(rebuildFileList, len(rebuildFileList), parityFileName)
                    if os.path.exists(parityFileName):
                        dhnio.Dprint(8, 'block_rebuilder.AttemptRebuild parity file found after rebuilding for supplier %d' % supplierNum)
                        self.localParity[supplierNum] = 1
            # so we have the parity on hand and it is missing - send it
            if self.localParity[supplierNum] == 1 and self.missingParity[supplierNum] == 1 and self.paritySent[supplierNum] == 0:
                dhnio.Dprint(8, 'block_rebuilder.AttemptRebuild new outstanding Parity for supplier %d' % supplierNum)
                self.outstandingFilesList.append((parityFileName, self.BuildFileName(supplierNum, 'Parity'), supplierNum))
                self.paritySent[supplierNum] = 1
        return True


    def SendFile(self, FileName, PacketID, supplierNum):
        io_throttle.QueueSendFile(FileName, 
                                  PacketID, 
                                  self.supplierSet.suppliers[supplierNum], 
                                  self.ownerId, 
                                  self.SendFileAcked, 
                                  self.SendFileFailed,)
        dhnio.Dprint(8, "block_rebuilder.SendFile to supplier %d" % supplierNum)


    def SendFileAcked(self, packet, remoteID, packetID):
        dhnio.Dprint(8, 'block_rebuilder.SendFileAcked to %s with %s' % (nameurl.GetName(remoteID), packetID))
        try:
            backupID, blockNum, supplierNum, dataORparity = packetID.split("-")
            blockNum = int(blockNum)
            supplierNum = int(supplierNum)
            if dataORparity == 'Data':
                self.missingData[supplierNum] = 0
                self.remoteData[supplierNum] = 1
                self.dataSent[supplierNum] = 2
            elif dataORparity == 'Parity':
                self.missingParity[supplierNum] = 0
                self.remoteParity[supplierNum] = 1
                self.paritySent[supplierNum] = 2
            else:
                dhnio.Dprint(8, "block_rebuilder.SendFileAcked WARNING odd PacketID? -" + str(packetID))
            if self.sentReports.has_key(packetID):
                dhnio.Dprint(2, 'block_rebuilder.SendFileAcked WARNING already got sending report for %s' % packetID)
            self.sentReports[packetID] = 'acked'
        except:
            dhnio.DprintException()
        self.automat('file-sent-report')


    def SendFileFailed(self, creatorID, packetID, reason):
        dhnio.Dprint(8, 'block_rebuilder.SendFileFailed to %s with %s (%s)' % (nameurl.GetName(creatorID), packetID, reason))
        try:
            backupID, blockNum, supplierNum, dataORparity = packetID.split("-")
            blockNum = int(blockNum)
            supplierNum = int(supplierNum)
            if dataORparity == 'Data':
                self.missingData[supplierNum] = 1
                self.remoteData[supplierNum] = 0
                self.dataSent[supplierNum] = 3
            elif dataORparity == 'Parity':
                self.missingParity[supplierNum] = 1
                self.remoteParity[supplierNum] = 0
                self.paritySent[supplierNum] = 3
            else:
                dhnio.Dprint(8, "block_rebuilder.SendFileFailed WARNING odd PacketID? -" + str(packetID))
            if self.sentReports.has_key(packetID):
                dhnio.Dprint(2, 'block_rebuilder.SendFileFailed WARNING already got sending report for %s' % packetID)
            self.sentReports[packetID] = reason
        except:
            dhnio.DprintException()
        self.automat('file-sent-report')


    def FileReceived(self, packet):
        try:
            self.requestedFilesList.remove(packet.PacketID)
        except:
            dhnio.DprintException()
            return
        # filename = os.path.join(tmpfile.subdir('data-par'), packet.PacketID)
        filename = os.path.join(settings.getLocalBackupsDir(), packet.PacketID)
        if packet.Valid():
            dhnio.Dprint(8, "block_rebuilder.FileReceived %s,  requestedFilesList length is %d" % (packet.PacketID, len(self.requestedFilesList)))
            # if we managed to rebuild a file 
            # before a request came in don't overwrite it
            if os.path.exists(filename):
                dhnio.Dprint(4, "block_rebuilder.FileReceived WARNING file already existed " + filename)
                try: 
                    os.remove(filename)
                except:
                    dhnio.DprintException()
            dhnio.WriteFile(filename, packet.Payload)
            try: 
                supplierNum = int((packet.PacketID.split("-"))[2])
                if packet.PacketID.endswith("-Data"):
                    self.localData[supplierNum] = 1
                elif packet.PacketID.endswith("-Parity"):
                    self.localParity[supplierNum] = 1
            except:
                dhnio.DprintException()              
        else:
            # TODO 
            # if we didn't get a valid packet ... re-request it or delete it?
            dhnio.Dprint(8, "block_rebuilder.FileReceived WARNING" + filename + " not valid")
            try: 
                os.remove(filename)
            except:
                dhnio.DprintException()
            #io_throttle.QueueRequestFile(self.FileReceived, misc.getLocalID(), packet.PacketID, self.ownerId, packet.RemoteID)


    def BuildRaidFileName(self, supplierNumber, dataOrParity):
        # return os.path.join(tmpfile.subdir('data-par'), self.BuildFileName(supplierNumber, dataOrParity))
        return os.path.join(settings.getLocalBackupsDir(), self.BuildFileName(supplierNumber, dataOrParity))


    def BuildFileName(self, supplierNumber, dataOrParity):
        return self.backupID + "-" + str(self.blockNum) + "-" + str(supplierNumber) + "-" + dataOrParity

#------------------------------------------------------------------------------ 



#!/usr/bin/python
#install.py
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#

import os
import sys

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in install.py')

from twisted.internet.defer import Deferred


import lib.dhnio as dhnio
import lib.dhncrypto as dhncrypto
import lib.dhnnet as dhnnet
import lib.misc as misc
import lib.settings as settings
import lib.identitycache as identitycache
import lib.nameurl as nameurl
import lib.stun as stun
import lib.identity as identity
import lib.transport_control as transport_control
import lib.automats as automats

import shutdowner
import identitypropagate
import central_service
import run_upnpc


DEBUG_PASS_STUN = False
DEBUG_PASS_SEND_IDENTITY = False
DEBUG_PASS_CACHE_CENTRAL_ID = False

MAX_WAITING_COUNT = 30

_Waiting4NameCount = 0
_Waiting4NameCall = None
_ProgressNotifyCallback = None
_RegistrationInProcess = False
_WorkingProtocols = []
_SentPackets = {}

#-------------------------------------------------------------------------------


def SetProgressNotifyCallback(cb):
    global _ProgressNotifyCallback
    _ProgressNotifyCallback = cb

def ProgressMessage(msg):
    global _ProgressNotifyCallback
    if _ProgressNotifyCallback is not None:
        _ProgressNotifyCallback(msg)

# return the result in the defer.callback (errback never will be called)
# possible values: exist, success, failed, timeout
def RegisterNewUser(login):
    global _RegistrationInProcess
    if _RegistrationInProcess:
        dhnio.Dprint(1, 'install.RegisterNewUser ERROR registration in process')
        return None
    _RegistrationInProcess = True
    dhnio.Dprint(4, 'install.RegisterNewUser ' + login)
    dhnio.WriteFile(settings.UserNameFilename(), login)
    url = nameurl.UrlMake('http', settings.IdentityServerName(), '', login+'.xml')
    identitycache.Clear()
    dhnio.Dprint(4, 'install.RegisterNewUser request ' + url)
    reactor.callLater(0, ProgressMessage, 'checking account name')
    res = dhnnet.getPageTwisted(url)
    finalDefer = Deferred()
    res.addCallback(username_busy, login, finalDefer)
    res.addErrback(username_free, login, finalDefer)
    return finalDefer

def username_busy(x, login, finalDefer):
    global _RegistrationInProcess
    dhnio.Dprint(4, 'install.username_busy')
    _RegistrationInProcess = False
    finalDefer.callback('exist')
    reactor.callLater(1, ProgressMessage, 'user already exist')

def username_free(x, login, finalDefer):
    dhnio.Dprint(4, 'install.username_free')
    reactor.callLater(0, ProgressMessage, 'detecting local IP address')
    reactor.callLater(1, detect_local_ip, login, finalDefer)

def detect_local_ip(login, finalDefer):
    dhnio.Dprint(4, 'install.detect_local_ip')
    if os.path.isfile(settings.LocalIPFilename()):
        localip = dhnio.ReadTextFile(settings.LocalIPFilename())
        reactor.callLater(0, ProgressMessage, 'your local IP is %s' % localip)
        reactor.callLater(1, ProgressMessage, 'detecting external IP address')
        reactor.callLater(2, detect_external_ip, login, finalDefer)
        return
    localip = dhnnet.getLocalIp()
    dhnio.WriteFile(settings.LocalIPFilename(), str(localip))
    reactor.callLater(0, ProgressMessage, 'your local IP is %s' % localip)
    reactor.callLater(1, ProgressMessage, 'detecting external IP address')
    reactor.callLater(2, detect_external_ip, login, finalDefer)

def detect_external_ip(login, finalDefer):
    dhnio.Dprint(4, 'install.detect_external_ip')
    if os.path.isfile(settings.ExternalIPFilename()):
        reactor.callLater(1, stun_success, dhnio.ReadTextFile(settings.ExternalIPFilename()), login, finalDefer)
        return
    if DEBUG_PASS_STUN:
        reactor.callLater(5, stun_success, '127.0.0.1', login, finalDefer)
        return
    d = stun.stunExternalIP(close_listener=False, internal_port=settings.getUDPPort())
    d.addCallback(stun_success, login, finalDefer)
    d.addErrback(stun_failed, login, finalDefer)

def stun_success(ip, login, finalDefer):
    dhnio.Dprint(4, 'install.stun_success ' + ip)
    dhnio.WriteFile(settings.ExternalIPFilename(), ip)
    reactor.callLater(0, ProgressMessage, 'your public IP is %s' % ip)
    if DEBUG_PASS_CACHE_CENTRAL_ID:
        reactor.callLater(1, cache_central_id_success, None, ip, login, finalDefer)
        return
    reactor.callLater(1, ProgressMessage, 'downloading central server identity')
    reactor.callLater(2, cache_central_id, ip, login, finalDefer)

def stun_failed(x, login, finalDefer):
    global _RegistrationInProcess
    dhnio.Dprint(4, 'install.stun_failed')
    _RegistrationInProcess = False
    reactor.callLater(0, ProgressMessage, 'connection error')
    reactor.callLater(1, registration_failed, finalDefer)

def cache_central_id(ip, login, finalDefer):
    dhnio.Dprint(4, 'install.cache_central_id')
    res = identitycache.immediatelyCaching(settings.CentralID())
    res.addCallback(cache_central_id_success, ip, login, finalDefer)
    res.addErrback(cache_central_id_failed, ip, login, finalDefer)

def cache_central_id_success(x, ip, login, finalDefer):
    dhnio.Dprint(4, 'install.cache_central_id_success')
    reactor.callLater(1, ProgressMessage, 'making the private key and identity file')
    reactor.callLater(2, make_identity, finalDefer, login, ip, 'upnp-disabled', 'upnp-disabled', 'upnp-disabled')
#    if not settings.enableUPNP():
#        reactor.callLater(1, ProgressMessage, 'making the private key and identity file')
#        reactor.callLater(2, make_identity, finalDefer, login, ip, 'upnp-disabled', 'upnp-disabled', 'upnp-disabled')
#    else:
#        reactor.callLater(1, ProgressMessage, 'configuring UPnP device')
#        reactor.callLater(2, install_upnp, ip, login, finalDefer)

def cache_central_id_failed(x, ip, login, finalDefer):
    global _RegistrationInProcess
    dhnio.Dprint(4, 'install.cache_central_id_failed WARNING')
    _RegistrationInProcess = False
    dhnio.restore_and_remove(settings.LocalIdentityFilename(), True)
    reactor.callLater(0, ProgressMessage, 'connection error')
    reactor.callLater(1, registration_failed, finalDefer)

def install_upnp(ip, login, finalDefer):
    dhnio.Dprint(4, 'install.install_upnp ' + ip)
    successTCP = 'upnp-not-found'
    successSSH = 'upnp-not-found'
    successHTTP = 'upnp-not-found'
    if settings.enableTCP():
        shutdowner.A('block')
        successTCP, tcp_port = run_upnpc.update(settings.getTCPPort())
        shutdowner.A('unblock')
        settings.setTCPPort(tcp_port)
#    if settings.enableSSH():
#        successSSH, ssh_port = run_upnpc.update(settings.getSSHPort())
#        settings.setSSHPort(ssh_port)
    if settings.enableHTTP():
        shutdowner.A('block')
        successHTTP, http_port = run_upnpc.update(settings.getHTTPPort())
        shutdowner.A('unblock')
        settings.setHTTPPort(http_port)
    reactor.callLater(0, ProgressMessage, 'making the private key and identity file')
    reactor.callLater(1, make_identity, finalDefer, login, ip, successTCP, successSSH, successHTTP)

def make_identity(finalDefer, login, ip, successTCP, successSSH, successHTTP):
    dhnio.Dprint(4, 'install.make_identity %s %s ' % (login, ip))
    dhnio.WriteFile(settings.ExternalIPFilename(), ip)
    ident = identity.makeDefaultIdentity(login, ip, successTCP, successSSH, successHTTP)
    misc.setLocalIdentity(ident)
    misc.saveLocalIdentity()
    dhnio.remove_backuped_file(settings.LocalIdentityFilename())
    dhnio.backup_and_remove(settings.LocalIdentityFilename())
#    reactor.callLater(0, ProgressMessage, 'propagate my identity')
#    reactor.callLater(1, send_identity, ip, login, finalDefer, False)
    reactor.callLater(0, ProgressMessage, 'starting transport protocols')
    reactor.callLater(1, install_transports, ip, login, finalDefer)

def inbox_packet(newpacket, proto, host):
    global _WorkingProtocols
    dhnio.Dprint(6, 'install.inbox_packet %s from %s://%s' % (newpacket.Command, proto, host))
    if proto not in _WorkingProtocols:
        _WorkingProtocols.append(proto)
    return True

def install_transports(ip, login, finalDefer):
    global _WorkingProtocols
    dhnio.Dprint(4, 'install.install_transports %s %s ' % (login, ip))
    def _init_callback():
        reactor.callLater(0, ProgressMessage, 'update local identity')
        reactor.callLater(1, update_identity, ip, login, finalDefer)
    _WorkingProtocols = []
    transport_control.AddInboxCallback(inbox_packet)
    transport_control.shutdown().addBoth(
        lambda x: transport_control.init(_init_callback))

def update_identity(ip, login, finalDefer, working_protos=None, waiting=True):
    dhnio.Dprint(4, 'install.update_identity')
    lid = misc.getLocalIdentity()
    if settings.enableCSpace() and settings.getCSpaceKeyID() != '':
        lid.setProtoContact('cspace', 'cspace://'+settings.getCSpaceKeyID())
    if settings.enableQ2Q() and settings.getQ2Qusername() != '' and settings.getQ2Qhost() != '':
        lid.setProtoContact('q2q', 'q2q://'+settings.getQ2Quserathost())
    contacts = lid.getContacts()
    first = lid.getContactProto(0)
    for c in contacts:
        proto, x, x, x = nameurl.UrlParse(c)
        if not transport_control.ProtocolIsSupported(proto):
            dhnio.Dprint(4, 'install.update_identity want to remove %s contact' % proto)
            lid.deleteProtoContact(proto)
    if first and working_protos and len(working_protos) > 0 and first not in working_protos:
        wantedproto = working_protos[0]
        if first != 'q2q' and 'q2q' in working_protos:
            wantedproto = 'q2q'
        if first != 'cspace' and 'cspace' in working_protos:
            wantedproto = 'cspace'
        if first != 'tcp' and 'tcp' in working_protos:
            wantedproto = 'tcp'
        dhnio.Dprint(4, 'install.update_identity want to pop %s contact ' % wantedproto)
        lid.popProtoContact(wantedproto)
    misc.setLocalIdentity(lid)
    misc.saveLocalIdentity()
    identitycache.Remove(lid.getIDURL())
    reactor.callLater(0, ProgressMessage, 'sending your identity')
    reactor.callLater(1, send_identity, ip, login, finalDefer, waiting)

def send_identity(ip, login, finalDefer, waiting):
    def _timeout(finalDefer, DL):
        if not DL.called:
            DL.errback('failed')
    dhnio.Dprint(4, 'install.send_identity')
    if DEBUG_PASS_SEND_IDENTITY:
        reactor.callLater(1, send_identity_success, None, ip, login, finalDefer)
    else:
        dl = identitypropagate.SendServers()
        dl.addCallback(send_identity_success, ip, login, finalDefer, waiting)
        dl.addErrback(send_identity_failed, ip, login, finalDefer)
        timeoutCall = reactor.callLater(30, _timeout, finalDefer, dl)

def send_identity_success(x, ip, login, finalDefer, waiting):
    global _Waiting4NameCount
    global _Waiting4NameCall
    dhnio.Dprint(4, 'install.send_identity_success')
    if waiting:
        _Waiting4NameCount = 0
        reactor.callLater(0, ProgressMessage, 'waiting response from identity server')
        _Waiting4NameCall = reactor.callLater(1, waiting4name, ip, login, finalDefer)
        return
    reactor.callLater(0, finish_new_identity, ip, login, finalDefer)

def send_identity_failed(x, ip, login, finalDefer):
    global _RegistrationInProcess
    dhnio.Dprint(4, 'install.send_identity_failed')
    _RegistrationInProcess = False
    reactor.callLater(0, ProgressMessage, 'connection error')
    reactor.callLater(1, registration_failed, finalDefer)

def waiting4name(ip, login, finalDefer):
    global _Waiting4NameCount
    global _Waiting4NameCall
    global _RegistrationInProcess
    dhnio.Dprint(4, 'install.waiting4name ' + str(_Waiting4NameCount))
    url = misc.getLocalIdentity().getIDURL()
    identitycache.scheduleForCaching(url)
    if not identitycache.HasKey(url):
        _Waiting4NameCount += 1
        if _Waiting4NameCount > MAX_WAITING_COUNT:
            _Waiting4NameCount = 0
            reactor.callLater(0, finish_new_identity, ip, login, finalDefer)
            return
        _Waiting4NameCall = reactor.callLater(1, waiting4name, ip, login, finalDefer)
        return
    xmlsrc1 = identitycache.FromCache(url).serialize()
    xmlsrc2 = misc.getLocalIdentity().serialize()
    if xmlsrc1 != xmlsrc2:
        _RegistrationInProcess = False
        _Waiting4NameCount = 0
        #transport_control.shutdown().addBoth(lambda x: finalDefer.callback('exist'))
        finalDefer.callback('exist')
        return
    _Waiting4NameCount = 0
    reactor.callLater(1, finish_new_identity, ip, login, finalDefer)

def finish_new_identity(ip, login, finalDefer):
    dhnio.Dprint(4, 'install.finish_new_identity')
    reactor.callLater(0, registration_done, finalDefer)
#    reactor.callLater(0, ProgressMessage, 'connecting to central server')
#    reactor.callLater(1, connect2central, ip, login, finalDefer)

def connect2central(ip, login, finalDefer):
    dhnio.Dprint(4, 'install.connect2central')
    def control(state, data):
        global _WorkingProtocols
        dhnio.Dprint(6, 'install.connect2central.control %s' % state)
        msg = state
        if state == 'Hello':
            msg = '[Identity] sent'
        elif state == 'HelloAck':
            msg = '[Ack] received'
            firstProto = misc.getLocalIdentity().getContactProto(0)
            if firstProto not in _WorkingProtocols:
                working_protos_copy = list(_WorkingProtocols)
                _WorkingProtocols = []
                identitycache.Clear([settings.CentralID()])
                reactor.callLater(0, ProgressMessage, '[Ack] received')
                reactor.callLater(1, ProgressMessage, 'update local identity')
                reactor.callLater(2, update_identity, ip, login, finalDefer, working_protos_copy)
                return False
        elif state == 'Settings':
            msg = '[Settings] sent'
        elif state == 'SettingsAck':
            msg = '[Ack] received'
            reactor.callLater(1, registration_done, finalDefer)
        elif state == 'RequestSettings':
            msg = '[RequestSettings] sent'
        elif state == 'RequestSettingsAck':
            msg = '[Ack] received'
        elif state == 'RequestSuppliers':
            msg = '[RequestSuppliers] sent'
        elif state == 'ListContacts':
            msg = '[ListContacts] received'
            reactor.callLater(1, registration_done, finalDefer)
        elif state == 'ListContactsError':
            msg = 'incorrect packet [ListContacts] received'
            reactor.callLater(1, registration_failed, finalDefer)
        elif state == 'Failed':
            msg = 'central server not responding'
            reactor.callLater(5, registration_failed, finalDefer)
        #reactor.callLater(0, ProgressMessage, msg)
        ProgressMessage(msg)
        return True
    central_service.SetControlFunc(control)
    central_service.shutdown().addBoth(lambda x: centralservice.init(10))

def registration_done(finalDefer):
    global _RegistrationInProcess
    dhnio.Dprint(4, 'install.registration_done')
    _RegistrationInProcess = False
    if not finalDefer.called:
        finalDefer.callback('success')
    reactor.callLater(1, ProgressMessage, 'registration done')

def registration_failed(finalDefer):
    global _RegistrationInProcess
    dhnio.Dprint(4, 'install.registration_failed')
    _RegistrationInProcess = False
    #transport_control.shutdown().addBoth(lambda x: finalDefer.callback('disconnected'))
    if not finalDefer.called:
        finalDefer.callback('disconnected')
    reactor.callLater(1, ProgressMessage, 'registration failed')

#------------------------------------------------------------------------------

# return the result in the defer.callback (errback never will be called)
# possible values:
#    remote_identity_not_valid,
#    invalid_identity_source
#    invalid_identity_url
#    remote_identity_bad_format
#    incorrect_key,
#    idurl_not_exist,
#    signing_error,
#    signature_not_match
#    central_failed,
#    success
def RecoverExistingUser(idurl, private_key_src):
    global _RegistrationInProcess
    if _RegistrationInProcess:
        dhnio.Dprint(1, 'install.RecoverExistingUser ERROR registration in process')
        return None
    _RegistrationInProcess = True
    dhnio.Dprint(4, 'install.RecoverExistingUser ' + idurl)
    finalDefer = Deferred()
    identitycache.Clear()
    res = dhnnet.getPageTwisted(idurl)
    res.addCallback(_recover, idurl, private_key_src, finalDefer)
    res.addErrback(_idurl_not_exist, finalDefer)
    return finalDefer

def _idurl_not_exist(x, finalDefer):
    global _RegistrationInProcess
    _RegistrationInProcess = False
    finalDefer.callback('idurl_not_exist')

def _recover(identity_src, idurl, private_key_src, finalDefer):
    global _RegistrationInProcess
    dhnio.Dprint(4, 'install._recover')

    xmlsrc = identity_src

    reactor.callLater(0, ProgressMessage, 'verification of the user account')

    dhnio.Dprint(4, 'install._recover will backup and remove ' + settings.KeyFileName())
    dhnio.backup_and_remove(settings.KeyFileName())

    dhnio.Dprint(4, 'install._recover will backup and remove ' + settings.LocalIdentityFilename())
    dhnio.backup_and_remove(settings.LocalIdentityFilename())

    try:
        remote_ident = identity.identity(xmlsrc=xmlsrc)
        local_ident = identity.identity(xmlsrc=xmlsrc)
    except:
        dhnio.Dprint(2, 'install._recover ERROR in the identity.__init__()')
        dhnio.DprintException()
#        dhnio.restore_and_remove(settings.KeyFileName(), True)
#        dhnio.restore_and_remove(settings.LocalIdentityFilename(), True)
        _RegistrationInProcess = False
        finalDefer.callback('remote_identity_bad_format')
        return

    try:
        res = remote_ident.Valid()
    except:
        dhnio.Dprint(2, 'install._recover ERROR in the identity.Valid()')
        dhnio.DprintException()
        res = False
    if not res:
#        dhnio.restore_and_remove(settings.KeyFileName(), True)
#        dhnio.restore_and_remove(settings.LocalIdentityFilename(), True)
        _RegistrationInProcess = False
        finalDefer.callback('remote_identity_not_valid')
        return

    dhnio.Dprint(4, 'install._recover will save the new private key')
    dhnio.WriteFile(settings.KeyFileName(), private_key_src)
    try:
        dhncrypto.InitMyKey()
    except:
        dhnio.Dprint(2, 'install._recover ERROR in the dhncrypto.InitMyKey()')
        dhnio.DprintException()
#        dhnio.restore_and_remove(settings.KeyFileName(), True)
#        dhnio.restore_and_remove(settings.LocalIdentityFilename(), True)
        _RegistrationInProcess = False
        finalDefer.callback('incorrect_key')
        return

    try:
        local_ident.sign()
    except:
        dhnio.Dprint(2, 'install._recover ERROR in the identity.sign()')
        dhnio.DprintException()
#        dhnio.restore_and_remove(settings.KeyFileName(), True)
#        dhnio.restore_and_remove(settings.LocalIdentityFilename(), True)
        _RegistrationInProcess = False
        finalDefer.callback('signing_error')
        return

    if remote_ident.signature != local_ident.signature:
        dhnio.Dprint(2, 'install._recover ERROR signature and private key did not match!!!')
#        dhnio.restore_and_remove(settings.KeyFileName(), True)
#        dhnio.restore_and_remove(settings.LocalIdentityFilename(), True)
        _RegistrationInProcess = False
        finalDefer.callback('signature_not_match')
        return

    misc.setLocalIdentity(local_ident)
    misc.saveLocalIdentity()

    dhnio.WriteFile(settings.UserNameFilename(), misc.getIDName())

    dhnio.remove_backuped_file(settings.KeyFileName())
    dhnio.remove_backuped_file(settings.LocalIdentityFilename())

    dhnio.Dprint(4, 'install._recover Done!')
    reactor.callLater(1, _get_central_id, finalDefer)

def _get_central_id(finalDefer):
    reactor.callLater(0, ProgressMessage, 'receiving the Central server Identity')
    res = identitycache.immediatelyCaching(settings.CentralID())
    res.addCallback(_central_id_success, finalDefer)
    res.addErrback(_central_id_failed, finalDefer)

def _central_id_failed(x, finalDefer):
    global _RegistrationInProcess
    _RegistrationInProcess = False
    dhnio.Dprint(4, 'install._central_id_failed WARNING Local Id cached but Central Id not? ...')
    finalDefer.callback('central_failed')

def _central_id_success(x, finalDefer):
    global _RegistrationInProcess
    _RegistrationInProcess = False
    finalDefer.callback('success')
    reactor.callLater(0, ProgressMessage, 'your account was restored!')

def usage():
    print 'usage:'
    print
    print '    install.py register  <username> [tcp port number]'
    print '    install.py recover   <private-key-filename>'
    print '    install.py local     <username>'
    print


def main():
    if len(sys.argv) < 3:
        usage()
        return

    settings.init()
    dhnio.SetDebug(18)


    def done(x):
        print
        print '!!! DONE !!!'
        print 'result is "%s"' % str(x)
        reactor.stop()
        #os._exit(0)


    def progress(s):
        print '[progress message]:', s

    if sys.argv[1] == 'recover':
        #test recovering
        src = dhnio.ReadTextFile(sys.argv[2])
        try:
            lines = src.split('\n')
            idurl = lines[0]
            txt = '\n'.join(lines[1:])
            fname = nameurl.UrlFilename(idurl)
            newurl = nameurl.FilenameUrl(fname)
            if idurl != newurl:
                idurl = ''
                txt = src
        except:
            idurl = ''
            txt = src
        print 'idurl:', idurl
        res = RecoverExistingUser(idurl, txt)
        res.addCallback(done)
        reactor.run()
        return

    elif sys.argv[1] == 'register':
        #test registering
        SetProgressNotifyCallback(progress)
        if len(sys.argv) > 3:
            tcpPort = int(sys.argv[3])
            settings.setTCPPort(tcpPort)
        print 'TCP port is:', settings.getTCPPort()
        res = RegisterNewUser(sys.argv[2])
        res.addCallback(done)
        reactor.run()
        return

    elif sys.argv[1] == 'local':
#        if len(sys.argv) > 3:
#            httpPort = int(sys.argv[3])
#            settings.setHTTPPort(httpPort)
#        print 'HTTP port is:', settings.getHTTPPort()
        ident = identity.makeDefaultIdentity(sys.argv[2])
        misc.setLocalIdentity(ident)
        misc.saveLocalIdentity()
        return

    usage()



if __name__ == '__main__':
    main()







#!/usr/bin/python
#dhnvertex.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#

import os
import sys
import struct
import random
import tempfile
import string
import time


if __name__ == '__main__':
    sys.path.append(os.path.abspath(os.path.join('..','..')))
    sys.path.append(os.path.abspath('..'))
    sys.path.append(os.path.abspath('datahaven'))


try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in dhnvertex.py')


from twisted.internet import protocol
from twisted.internet.task import LoopingCall
from twisted.application import service as service_app
from twisted.internet.defer import Deferred, fail
from twisted.internet.base import DelayedCall
from twisted.protocols.basic import FileSender as fsdr
DelayedCall.debug = False


import vertex.q2qclient as q2qclient
import vertex.q2q as q2q

#------------------------------------------------------------------------------

_Q2QDir = os.path.join('.', '.q2qcerts')
_Service = None
_LogFunc = None
_OpenedPorts = {}

_ReceiveNotifyFunc = None
_SentNotifyFunc = None
_TempFileFunc = None

#------------------------------------------------------------------------------

def init(
         q2qdir=os.path.join('.', '.q2qcerts'),
         logFunc=None,
         receiveNotifyFunc=None,
         sentNotifyFunc=None,
         tempFileFunc=None,
         ):

    global _Q2QDir
    global _LogFunc
    global _ReceiveNotifyFunc
    global _SentNotifyFunc
    global _TempFileFunc

    _Q2QDir = q2qdir
    if logFunc is not None:
        _LogFunc = logFunc
    if receiveNotifyFunc is not None:
        _ReceiveNotifyFunc = receiveNotifyFunc
    if sentNotifyFunc is not None:
        _SentNotifyFunc = sentNotifyFunc
    if tempFileFunc is not None:
        _TempFileFunc = tempFileFunc

    logmsg(4, 'dhnvertex.init ' + _Q2QDir)

#    reactor.addSystemEventTrigger('before', 'shutdown', stopService)
    return startService()


def shutdown():
    logmsg(4, 'dhnvertex.shutdown ')
    return stopService()

def getDefaultPath():
    global _Q2QDir
    return os.path.join(_Q2QDir, 'default-address')

def getFrom():
    defpath = getDefaultPath()
    if os.path.exists(defpath):
        fr = file(defpath).read()
    else:
        fr = getService().getDefaultFrom()
        if fr is None:
            return None
        logmsg(4, 'dhnvertex.getFrom  %s set to default user' % fr)
        f = file(defpath, 'wb')
        f.write(fr)
        f.close()
    return q2q.Q2QAddress.fromString(fr)

def getService():
    global _Service
    global _Q2QDir
    if _Service is None:
        if _Q2QDir is None:
            _Q2QDir = os.path.join('.', '.q2qcerts')
        logmsg(8, 'dhnvertex.getService open local database in: %s' % _Q2QDir)
        _Service = q2qclient.ClientQ2QService(_Q2QDir)
    return _Service

def startService():
    global _DispatcherTask
    logmsg(6, 'dhnvertex.startService')
    svc = getService()
    R = service_app.IService(svc).startService()
    #install our Dispatcher
    prevDisp = svc.dispatcher
    svc.dispatcher = MyPTCPConnectionDispatcher(prevDisp.factory)
    svc.dispatcher._ports.update(prevDisp._ports)
    del prevDisp
    _DispatcherTask = LoopingCall(svc.dispatcher.collect_unused_ports)
    _DispatcherTask.start(10, False)
    return R

def stopService():
    global _Service
    global _DispatcherTask
    logmsg(6, 'dhnvertex.stopService')
    if _DispatcherTask is not None:
        if _DispatcherTask.running:
            _DispatcherTask.stop()
    svc = getService()
    _Service = None
    return service_app.IService(svc).stopService()

def register(user, password):
    logmsg(6, 'dhnvertex.register ' + user)
    newAddress = q2q.Q2QAddress.fromString(user)
    svc = getService()
    D = svc.connectQ2Q(q2q.Q2QAddress("",""),
                       q2q.Q2QAddress(newAddress.domain, "accounts"),
                       'identity-admin',
                       q2qclient.UserAdderFactory(
                           newAddress.resource, password))
    D.addCallback(lambda proto: svc.authorize(newAddress, password))
    return D

class FileReceiver(protocol.Protocol):
    gotLength = False
    def portNumber(self):
        try:
            return self.transport.getHost().logical.port
        except:
            return -1

    def connectionMade(self):
        global _TempFileFunc
        logmsg(12, 'dhnvertex.FileReceiver.connectionMade local port is %d' % self.portNumber())
        if _TempFileFunc is None:
            self.fd, self.filename = tempfile.mkstemp(".dhn-q2q-in")
        else:
            self.fd, self.filename = _TempFileFunc('q2q-in')
        port_update(self.portNumber())

    def dataReceived(self, data):
        if not self.gotLength:
            self.length ,= struct.unpack("!Q", data[:8])
            self.bytesReceived = 0
            data = data[8:]
            self.gotLength = True
            logmsg(12, 'dhnvertex.FileReceiver.dataReceived got length: ' + str(self.length))
        os.write(self.fd, data)
        self.bytesReceived += len(data)
        port_update(self.portNumber())

    def connectionLost(self, reason):
        global _ReceiveNotifyFunc
        logmsg(12, 'dhnvertex.FileReceiver.connectionLost local port is %d' % self.portNumber())
        os.fsync(self.fd)
        os.close(self.fd)
        status = 'finished'
        if self.bytesReceived != self.length:
            status = 'incomplete'
        if _ReceiveNotifyFunc is not None:
            _ReceiveNotifyFunc(self.transport.getPeer(), self.filename, status, reason)
        port_update(self.portNumber(), False)

class FileSender(protocol.Protocol):
    def portNumber(self):
        try:
            return self.transport.getHost().logical.port
        except:
            return -1

    def connectionMade(self):
        logmsg(12, 'dhnvertex.FileSender.connectionMade local port is %d' % self.portNumber())
        self.status = 'started'
        try:
            self.filename = self.factory.filename
            self.file = file(self.filename, 'rb')
            self.file.seek(0, 2)
            self.length = self.file.tell()
            self.file.seek(0)
        except:
            logmsg(1, 'dhnvertex.FileSender.connectionMade ERROR while opening the file. closing!')
            self.transport.loseConnection()
            return
        self.transport.write(struct.pack("!Q", self.length))
        d = fsdr().beginFileTransfer(self.file, self)
        d.addCallback(self.done)
        d.addErrback(self.failed)
        port_update(self.portNumber())

    def done(self, x):
        logmsg(12, 'dhnvertex.FileSender.done ')
        self.status = 'finished'
        self.transport.loseConnection()
        port_update(self.portNumber(), False)

    def failed(self, x):
        logmsg(10, 'dhnvertex.FileSender.failed NETERROR ' + str(x.getErrorMessage()))
        self.status = 'failed'
        self.transport.loseConnection()
        port_update(self.portNumber(), False)

    def dataReceived(self, data):
        logmsg(1, 'dhnvertex.FileSender.dataReceived ERROR THE CLIENT IS GETTING DATA from ' + str(self.transport.getPeer()))

    def registerProducer(self, producer, streaming):
        self.transport.registerProducer(producer, streaming)

    def unregisterProducer(self):
        self.transport.unregisterProducer()

    def write(self, data):
        self.transport.write(data)
        port_update(self.portNumber())

    def connectionLost(self, reason):
        global _SentNotifyFunc
        logmsg(12, 'dhnvertex.FileSender.connectionLost ' + str(reason))
        if _SentNotifyFunc is not None:
            _SentNotifyFunc(self.transport.getPeer(), self.filename, self.status, reason)
        port_update(self.portNumber(), False)

class FileSenderFactory(protocol.ClientFactory):
    protocol = FileSender
    def __init__(self, filename):
        self.filename = filename

class FileReceiverFactory(protocol.Factory):
    protocol = FileReceiver


def receive():
    fromAddress = getFrom()
    if fromAddress is None:
        logmsg(1, 'dhnvertex.receive ERROR no default user found. You must register first!')
        return fail(0)
    logmsg(6, 'dhnvertex.receive ' + str(getFrom()))

    service = getService()
    f = FileReceiverFactory()
    d = service.listenQ2Q( fromAddress, {'file-transfer': f}, 'file-transfer',)
    return d


def send(to_user, filename):
    fromAddress = getFrom()
    if fromAddress is None:
        logmsg(1, 'dhnvertex.send ERROR no default user found. You must register first!')
        return fail(0)

    toAddress = q2q.Q2QAddress.fromString(to_user)
    toDomain = toAddress.domainAddress()

    logmsg(12, 'dhnvertex.send  ['+str(fromAddress)+']->['+str(toAddress)+']')

    service = getService()
    f = FileSenderFactory(filename)
    d = service.connectQ2Q(fromAddress, toAddress, 'file-transfer', f, )
    return d

#-------------------------------------------------------------------------------

def port_update(portNum, InUse=True):
    global _OpenedPorts
    if portNum not in _OpenedPorts.keys():
        return
    if InUse:
        _OpenedPorts[portNum] = time.time()
    else:
        _OpenedPorts[portNum] = 0

class MyPTCPConnectionDispatcher(q2q.PTCPConnectionDispatcher):
    TIMEOUT = 60

    def bindNewPort(self, portNum=0):
        global _OpenedPorts
        new_port = q2q.PTCPConnectionDispatcher.bindNewPort(self, portNum)
        _OpenedPorts[new_port] = time.time()
        logmsg(12, 'dhnvertex.MyPTCPConnectionDispatcher.bindNewPort %d opened_ports=%d' % (new_port, len(_OpenedPorts)))
        return new_port

    def unbindPort(self, portNum):
        global _OpenedPorts
        logmsg(12, 'dhnvertex.MyPTCPConnectionDispatcher.unbindPort %d opened_ports=%d' % (portNum, len(_OpenedPorts)))
        if portNum not in _OpenedPorts.keys():
            logmsg(12, 'dhnvertex.MyPTCPConnectionDispatcher.unbindPort WARNING %d not in the opened ports' % portNum )
            return
        _OpenedPorts.pop(portNum, None)
        q2q.PTCPConnectionDispatcher.unbindPort(self, portNum)

    def collect_unused_ports(self):
        global _OpenedPorts
        logmsg(14, 'dhnvertex.MyPTCPConnectionDispatcher.collect_unused_ports opened=%d' % len(_OpenedPorts))
        removeL = []
        for port in _OpenedPorts.keys():
            if time.time() - _OpenedPorts[port] > self.TIMEOUT:
                removeL.append(port)
                continue

        for port in removeL:
            logmsg(12, 'dhnvertex.MyPTCPConnectionDispatcher.collect_unused_ports want to unbind port %d' % port)
            self.unbindPort(port)

        del removeL

#------------------------------------------------------------------------------

def logmsg(level, msg):
    global _LogFunc
    if _LogFunc is not None:
        _LogFunc(level, msg)
    else:
        print '%s%s' % (' '*level, msg)

def rmdir_recursive(dir):
    #http://mail.python.org/pipermail/python-list/2000-December/060960.html
    """Remove a directory, and all its contents if it is not already empty."""
    for name in os.listdir(dir):
        full_name = os.path.join(dir, name)
        # on Windows, if we don't have write permission we can't remove
        # the file/directory either, so turn that on
        if not os.access(full_name, os.W_OK):
            os.chmod(full_name, 0600)
        if os.path.isdir(full_name):
            rmdir_recursive(full_name)
        else:
            os.remove(full_name)
    os.rmdir(dir)

def rndstr(length):
    return ''.join([random.choice(string.letters+string.digits) for i in range(0,length)])

def clear_local_db(local_db_dir=None):
    global _Q2QDir
    if local_db_dir is not None:
        _Q2QDir = local_db_dir
    try:
        if os.path.isdir(_Q2QDir):
            rmdir_recursive(_Q2QDir)
            logmsg(8, 'dhnvertex.clear_local_db %s removed' % _Q2QDir)
    except:
        logmsg(1, 'dhnvertex.clear_local_db ERROR could not remove %s' % _Q2QDir)

#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------


def _test_receive():
    d = receive()
    d.addCallback(lambda x: logmsg(10, 'register receiveing success. now waiting connections ... '))
    d.addErrback(lambda x: logmsg(10, 'register receiving failed!!!' + str(x)))

def _test_send():
    to = sys.argv[2]
    filename = sys.argv[3]

    timeout = 0
    if len(sys.argv) > 4:
        timeout = int(sys.argv[4])

    def _do_send():
        D = send(to, filename)
        D.addCallback(lambda x: logmsg(10, 'connection success'))
        D.addErrback(lambda x: logmsg(10, ' connection failed!!!: '+str(x)))
        return D

    if timeout == 0:
        _do_send().addBoth(lambda x: reactor.stop())
        return

    t = LoopingCall(_do_send)
    logmsg(4, 'dhnvertex._test_send going to start sending %s in loop' % filename)
    t.start(timeout)

def _test_register():
    u = sys.argv[2]
    password = sys.argv[3]

    def ok(x, u, password):
        logmsg(8, 'registration done success: ' + u)
        reactor.stop()

    def fail(x):
        logmsg(8, 'registration failed: ' + str(x))
        reactor.stop()

    d = register(u, password)
    d.addCallback(ok, u, password)
    d.addErrback(fail)

def _print_help():
    print '\nusage:'
    print 'dhnvertex.py receive'
    print 'dhnvertex.py register <user@server> <password>'
    print 'dhnvertex.py send <to_user@server> <filename> [timeout]'
    print 'dhnvertex.py clear'

def _do_cmd(cmd):
    logmsg(4, 'dhnvertex._do_cmd [%s]' % cmd)
    if cmd == 'send':
        _test_send()

    elif cmd == 'receive':
        _test_receive()

    elif cmd == 'register':
        _test_register()

    elif cmd == 'clear':
        clear_local_db()
        sys.exit(0)

    else:
        _print_help()
        sys.exit(0)


if __name__ == '__main__':
    if len(sys.argv) < 2:
        _print_help()
        sys.exit(0)

    init()

    cmd = sys.argv[1].strip().lower()
    _do_cmd(cmd)

    reactor.run()






#!/usr/bin/python
#identitydb.py

import os
import dhnio
import settings
import identity
import nameurl

# Dictionary cache of identities - lookup by primary url
# global dictionary of identities in this file
# indexed with urls and contains identity objects
_IdentityCache = {}
_Contact2IDURL = {}
_IDURL2Contacts = {}

#------------------------------------------------------------------------------ 

def init():
    dhnio.Dprint(4,"identitydb.init")
    iddir = settings.IdentityCacheDir()
    if not os.path.exists(iddir):
        dhnio.Dprint(8, 'identitydb.init create folder ' + iddir)
        dhnio._dir_make(iddir)

def clear(exclude_list=None):
    global _IdentityCache
    global _Contact2IDURL
    global _IDURL2Contacts
    dhnio.Dprint(4,"identitydb.clear")
    _IdentityCache.clear()
    _Contact2IDURL.clear()
    _IDURL2Contacts.clear()

    iddir = settings.IdentityCacheDir()
    if not os.path.exists(iddir):
        return

    for name in os.listdir(iddir):
        path = os.path.join(iddir, name)
        if not os.access(path, os.W_OK):
            continue
        if exclude_list:
            idurl = nameurl.FilenameUrl(name)
            if idurl in exclude_list:
                continue 
        os.remove(path)
        dhnio.Dprint(6, 'identitydb.clear remove ' + path)

def size():
    global _IdentityCache
    return len(_IdentityCache)

def has_key(url):
    global _IdentityCache
    return _IdentityCache.has_key(url)

def idset(url, id_obj):
    global _IdentityCache
    global _Contact2IDURL
    global _IDURL2Contacts
    if not has_key(url):
        dhnio.Dprint(6, 'identitydb.idset new identity: ' + url)
    _IdentityCache[url] = id_obj
    for contact in id_obj.getContacts():
        if not _Contact2IDURL.has_key(contact):
            _Contact2IDURL[contact] = set()
        else:
            if len(_Contact2IDURL[contact]) >= 1 and url not in _Contact2IDURL[contact]:
                dhnio.Dprint(6, 'identitydb.idset WARNING another user have same contact: ' + str(list(_Contact2IDURL[contact])))
        _Contact2IDURL[contact].add(url)
        if not _IDURL2Contacts.has_key(url):
            _IDURL2Contacts[url] = set()
        _IDURL2Contacts[url].add(contact) 

def idget(url):
    global _IdentityCache
    return _IdentityCache.get(url, None)

def idremove(url):
    global _IdentityCache
    global _Contact2IDURL
    global _IDURL2Contacts
    idobj = _IdentityCache.pop(url, None)
    _IDURL2Contacts.pop(url, None)
    if idobj is not None:
        for contact in idobj.getContacts():
            _Contact2IDURL.pop(contact, None)
    return idobj

def idcontacts(idurl):
    global _IDURL2Contacts
    return list(_IDURL2Contacts.get(idurl, set()))

def get(url):
    if has_key(url):
        return idget(url)
    else:
        try:
            partfilename = nameurl.UrlFilename(url)
        except:
            dhnio.Dprint(1, "identitydb.get ERROR %s is incorrect" % str(url))
            return None
        
        filename = os.path.join(settings.IdentityCacheDir(), partfilename)
        if not os.path.exists(filename):
            dhnio.Dprint(6, "identitydb.get file %s not exist" % os.path.basename(filename))
            return None
        
        idxml = dhnio.ReadTextFile(filename)
        if idxml:
            idobj = identity.identity(xmlsrc=idxml)
            url2 = idobj.getIDURL()
            if url == url2:
                idset(url, idobj)
                return idobj
            
            else:
                dhnio.Dprint(1, "identitydb.get ERROR url=%s url2=%s" % (url, url2))
                return None

        dhnio.Dprint(6, "identitydb.get %s not found" % nameurl.GetName(url))
        return None

def get_idurls_by_contact(contact):
    global _Contact2IDURL
    return list(_Contact2IDURL.get(contact, set()))

def update(url, xml_src):
    try:
        newid = identity.identity(xmlsrc=xml_src)
    except:
        dhnio.DprintException()
        return False

    if not newid.isCorrect():
        dhnio.Dprint(1, "identitydb.update ERROR: incorrect identity " + str(url))
        return False

    try:
        if not newid.Valid():
            dhnio.Dprint(1, "identitydb.update ERROR identity not Valid" + str(url))
            return False
    except:
        dhnio.DprintException()
        return False

    filename=os.path.join(settings.IdentityCacheDir(), nameurl.UrlFilename(url))
    if os.path.exists(filename):
        oldidentityxml = dhnio.ReadTextFile(filename)
        oldidentity = identity.identity(xmlsrc=oldidentityxml)

        if oldidentity.publickey != newid.publickey:
            dhnio.Dprint(1, "identitydb.update ERROR new publickey does not match old : SECURITY VIOLATION " + url)
            return False

        if oldidentity.signature != newid.signature:
            dhnio.Dprint(6, 'identitydb.update have new data for ' + nameurl.GetName(url))
        else:
            idset(url, newid)
            return True

    # PREPRO need to check that date or version is after old one so not vulnerable to replay attacks
    dhnio.WriteFile(filename, xml_src)             # publickeys match so we can update it
    idset(url, newid)

    return True

def remove(url):
    filename = os.path.join(settings.IdentityCacheDir(), nameurl.UrlFilename(url))
    if os.path.isfile(filename):
        dhnio.Dprint(6, "identitydb.remove file %s" % filename)
        try:
            os.remove(filename)
        except:
            dhnio.DprintException()
    idremove(url)

#------------------------------------------------------------------------------ 

def print_id(url):
    if has_key(url):
##        print "has key"
##        idForKey = _IdentityCache[url]
        idForKey = get(url)
        dhnio.Dprint(6, str(idForKey.sources) )
        dhnio.Dprint(6, str(idForKey.contacts ))
        dhnio.Dprint(6, str(idForKey.publickey ))
        dhnio.Dprint(6, str(idForKey.signature ))

def print_keys():
    global _IdentityCache
    for key in _IdentityCache.keys():
        dhnio.Dprint(6, key)
##    print "PrintCacheKeys"

def print_cache():
    global _IdentityCache
##    print "PrintCacheKeys"
    for key in _IdentityCache.keys():
        dhnio.Dprint(6, "---------------------" )
        print_id(key)












#!/usr/bin/python
#backup_db.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#1)  This tracks backups in metadata/backup_info.xml.
#    It saves the directories to be backed up,
#    the schedule for being backed up if there is one,
#    and then the backups,
#    so when our suppliers say they have files for a particular backup (F20110718100000AM),
#    we know what directory that corresponds to.
#    It should also send the backup_info.xml to the suppliers so that data is backed up.
#    TODO it should encrypt the file and send it out.
#
#2)  Tracks backups in process so we don't try to fix a backup that isn't finished,
#    also so we don't delete a backup in process.



import os
import sys
import time
import locale
import xml.dom.minidom
#from xml.dom import Node

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor backup_db.py')

import lib.dhnio as dhnio
import lib.misc as misc
import lib.nameurl as nameurl
import lib.settings as settings
import lib.dirsize as dirsize
import lib.commands as commands
import lib.transport_control as transport_control
import lib.dhnpacket as dhnpacket
import lib.contacts as contacts
import lib.dhncrypto as dhncrypto
import lib.schedule as schedule


class _BackupRun:
    def __init__(self, backupID, backupSize=0, backupStatus="", backupStart="", backupFinish=""):
        self.backupID = backupID
        self.backupSize = backupSize
        self.backupStatus = backupStatus
        self.backupStart = backupStart
        self.backupFinish = backupFinish
        if str(self.backupStart) == "":
            self.backupStart = str(time.time())

    def SetStatus(self, backupStatus, backupFinish):
        self.backupStatus = backupStatus
        self.backupFinish = backupFinish

    def GetRunInfo(self):
        return self.backupID, self.backupSize, self.backupStatus, self.backupStart, self.backupFinish

    def Show(self):
        return "  " + self.backupID + "-" + self.backupStatus + "-" + self.backupSize + "-" + self.backupStart + "-" + self.backupFinish

    def ToXML(self):
        runXML =  "      <backupRun>\n"
        runXML += "        <backupId>" + self.backupID + "</backupId>\n"
        runXML += "        <backupSize>" + str(self.backupSize) + "</backupSize>\n"
        runXML += "        <backupStatus>" + self.backupStatus + "</backupStatus>\n"
        runXML += "        <backupStart>" + str(self.backupStart) + "</backupStart>\n"
        runXML += "        <backupFinish>" + str(self.backupFinish) + "</backupFinish>\n"
        runXML += "      </backupRun>\n"
        return runXML


class _BackupSchedule:
    def __init__(self, sched):
        self.schedule = sched
#        self.scheduleType = scheduleType
#        self.scheduleTime = scheduleTime
#        self.scheduleInterval = scheduleInterval
#        self.intervalDetails = intervalDetails

    def SetSchedule(self, sched):
        self.schedule = sched
#        self.scheduleType = scheduleType
#        self.scheduleTime = scheduleTime
#        self.scheduleInterval = scheduleInterval
#        self.intervalDetails = intervalDetails

    def Show(self):
#        return "  "  + self.scheduleType + "-" + self.scheduleTime + "-" + str(self.scheduleInterval) + "-" + self.intervalDetails
        return self.schedule.description()

    def ToXML(self):
#        scheduleXML =  "    <backupSchedule>\n"
#        scheduleXML += "      <scheduleType>" + str(self.scheduleType) + "</scheduleType>\n"
#        scheduleXML += "      <scheduleTime>" + self.scheduleTime + "</scheduleTime>\n"
#        scheduleXML += "      <scheduleInterval>" + str(self.scheduleInterval) + "</scheduleInterval>\n"
#        scheduleXML += "      <intervalDetails>" + self.intervalDetails + "</intervalDetails>\n"
#        scheduleXML += "    </backupSchedule>\n"
        scheduleXML =  "    <backupSchedule>\n"
        scheduleXML += "      <scheduleType>" + self.schedule.type + "</scheduleType>\n"
        scheduleXML += "      <scheduleTime>" + self.schedule.daytime + "</scheduleTime>\n"
        scheduleXML += "      <scheduleInterval>" + self.schedule.interval + "</scheduleInterval>\n"
        scheduleXML += "      <intervalDetails>" + self.schedule.details + "</intervalDetails>\n"
        scheduleXML += "    </backupSchedule>\n"
        return scheduleXML


class _BackupDirectory:
    def __init__(self, dirName, backupSubdirectories=True):
        self.dirName = dirName
        self.backupSubdirectories = backupSubdirectories # do subdirectories
        self.schedule = None
        self.backupRuns = []  # an array of _BackupRun in order of time, oldest to newest
        self.backupObject = None
        self.runningBackup = False

    def AddBackupRun(self, backupID, backupSize, backupStatus, backupStart, backupFinish, initialLoad=False):
        if initialLoad and backupStatus in ['in process', 'sending']:
            self.backupRuns.append(_BackupRun(backupID, backupSize, "unknown", backupStart, backupFinish))
        else:
            self.backupRuns.append(_BackupRun(backupID, backupSize, backupStatus, backupStart, backupFinish))
            if backupStatus in ['in process', 'sending']:
                self.runningBackup = True

    def DeleteDirBackup(self, backupID):
        delBackupRun = None
        for backupRun in self.backupRuns:
            if backupRun.backupID == backupID:
                delBackupRun = backupRun
        if delBackupRun != None:
            self.backupRuns.remove(delBackupRun)

    def GetLastRunInfo(self):
        if len(self.backupRuns)>0:
            return self.backupRuns[-1].GetRunInfo()
        else:
            return "", "0.0", "(none)", str(time.time()), ""

    def GetDirBackupIds(self):
        backupIds = []
        for backupRun in self.backupRuns:
            backupIds.append(str(backupRun.backupID))
        return backupIds

    def GetDirectorySubfoldersInclude(self):
        return self.backupSubdirectories

    def SetDirectorySubfoldersInclude(self, backupSubdirectories):
        if backupSubdirectories == False:
            self.backupSubdirectories = False
        else:
            self.backupSubdirectories = True

    def IsBackupRunning(self):
        return self.runningBackup

    def SetBackupStatus(self, backupID, backupStatus, backupFinish):
        for backupRun in self.backupRuns:  #TODO, do we need backupID at all?  Just set backupRuns[-1] ?
            if backupRun.backupID == backupID:
                dhnio.Dprint(6, 'backup_db.SetBackupStatus %s %s %s' % (backupID, backupStatus, str(backupFinish)))
                backupRun.SetStatus(backupStatus, backupFinish)
                self.runningBackup = False
                return

    def GetDirectoryInfo(self):
        if len(self.backupRuns) > 0:
            mostRecentBackupId = ""
            totalSize = 0.0
            mostRecentStatus = ""
            for backupRun in self.backupRuns:
                backupID, backupSize, backupStatus, backupStart, backupFinish = backupRun.GetRunInfo()
                totalSize += int(backupSize)
                mostRecentBackupId = backupID
                mostRecentStatus = backupStatus
            return mostRecentBackupId, totalSize, mostRecentStatus
        else:
            return '', 0, ''

    def SetDirSchedule(self, schedule):
        if self.schedule == None:
            self.schedule = _BackupSchedule(schedule)
        else:
            self.schedule.SetSchedule(schedule)

    def Show(self):
        dirShow =  self.dirName + "\n"
        if self.schedule:
            dirShow = dirShow + self.schedule.Show() + "\n"
        for backupRun in self.backupRuns:
            dirShow = dirShow + backupRun.Show() + "\n"
        return dirShow

    def ToXML(self):
        dirXML =  "  <backupDirectory>\n"
        dirXML += "    <directoryName>" + self.dirName + "</directoryName>\n"
        dirXML += "    <backupSubdirectories>" + str(self.backupSubdirectories) + "</backupSubdirectories>\n"
        if self.schedule:
            dirXML += self.schedule.ToXML()
        if len(self.backupRuns) > 0:
            dirXML += "    <directoryBackups>\n"
            for backupRun in self.backupRuns:
                dirXML += backupRun.ToXML()
            dirXML += "    </directoryBackups>\n"
        dirXML += "  </backupDirectory>\n"
        return dirXML


class _BackupDB:
    def __init__(self):
        self.backupDirs = {}
        self.currentlyRunningBackups = {}
        #self._oldBackupsDir = os.path.join(os.path.expanduser("~"),"datahavennet","backups")
        self._xmlFile = settings.BackupInfoFileFullPath()
        self.deletedBackups = []
        self._maxDeleted = 50 # how many deleted backupIDs do we want to hold on to?
        self.localID = misc.getLocalID()
        self._loading = False

    def BackupDirs(self):
        return self.backupDirs

    def AskSizeForAllDirs(self):
        for backupDir in self.backupDirs.keys():
            dirsize.ask(backupDir)

    def _GetText(self, nodelist): # get text from DOM elements
        rc = ""
        for node in nodelist:
            if node.hasChildNodes():
                rc = rc + self._GetText(node.childNodes)
            elif node.nodeType == node.TEXT_NODE: # TEXT_NODE is node type 3
                rc = rc + node.data.strip()
        return rc

    def _LoadXML(self):
        try:
            xmlDOM = xml.dom.minidom.parse(self._xmlFile)
        except:
            dhnio.Dprint(2, "backup_db._LoadXML ERROR not a valid XML file")
            return

        rootNode = xmlDOM.documentElement
        backupDirs = rootNode.getElementsByTagName("backupDirectory")
        for backupDir in backupDirs: # DOM Elements
            self._LoadDir(backupDir)
        deletedBackups = rootNode.getElementsByTagName("deletedBackup")
        for deletedBackup in deletedBackups:
            self._LoadDeleted(deletedBackup)
            #self.deletedBackups.append(self._GetText(deletedBackup))

    def _LoadDeleted(self, deletedBackup): # backupDir is some DOM Elements
        backupID = str(self._GetText(deletedBackup.getElementsByTagName("deletedBackupId")))
        #print "backupID="+backupID
        if backupID not in self.deletedBackups:
            self.deletedBackups.append(backupID)

    def _LoadDir(self, backupDir): # backupDir is some DOM Elements
        dirName = str(self._GetText(backupDir.getElementsByTagName("directoryName")))
        backupSubdirectories = str(self._GetText(backupDir.getElementsByTagName("backupSubdirectories")))
        if backupSubdirectories == "False":
            self.AddDirectory(dirName, False, initialLoad=True)
        else:
            self.AddDirectory(dirName, True, initialLoad=True)
        for schedule in backupDir.getElementsByTagName("backupSchedule"):
            self._LoadSchedule(dirName, schedule) # schedule is some DOM elements
        for backupRun in backupDir.getElementsByTagName("backupRun"):
            self._LoadBackupRun(dirName, backupRun) # backupRun is some DOM elements)

    def _LoadBackupRun(self, dirName, backupRun):
        self.AddDirBackup(dirName, 
                          str(self._GetText(backupRun.getElementsByTagName("backupId"))), \
                          str(self._GetText(backupRun.getElementsByTagName("backupStatus"))), \
                          misc.ToInt(self._GetText(backupRun.getElementsByTagName("backupSize"))), \
                          misc.ToFloat(self._GetText(backupRun.getElementsByTagName("backupStart"))), \
                          misc.ToFloat(self._GetText(backupRun.getElementsByTagName("backupFinish"))), \
                          initialLoad=True)

    def _LoadSchedule(self, dirName, sched):
        self.SetDirSchedule(dirName, schedule.Schedule( 
                            typ = str(self._GetText(sched.getElementsByTagName("scheduleType"))), 
                            daytime = str(self._GetText(sched.getElementsByTagName("scheduleTime"))), 
                            interval = str(self._GetText(sched.getElementsByTagName("scheduleInterval"))), 
                            details = str(self._GetText(sched.getElementsByTagName("intervalDetails")))))

    def Save(self):
##        print 'Save'
        if self._loading:
            return
        try:
            src = self.ToXML()
            f = open(self._xmlFile+"new.xml", "w")
            f.write(src)
            f.close()
            try:
                xmlDOM = xml.dom.minidom.parse(self._xmlFile+"new.xml")
                # make sure we are saving something valid
            except:
                dhnio.Dprint(2, "backup_db.Save ERROR new file is not a valid XML file")
                dhnio.DprintException()
                return

            if dhnio.Windows() and os.path.exists(self._xmlFile):
                os.remove(self._xmlFile)
            os.rename(self._xmlFile+"new.xml",self._xmlFile)

            #f = open(str(time.time())+"backup.xml","w") # used during development to debug some stuff
            #f.write(self.ToXML())
            #f.close()
            Payload = dhnio.ReadBinaryFile(self._xmlFile)
##            for supplierId in contactsdb.suppliers():
            def _send( Payload, localID):
                for supplierId in contacts.getSupplierIDs():
                    dhnio.Dprint(8, "backup_db.Save backing up " + settings.BackupInfoFileName() + " to " + nameurl.GetName(supplierId))
                    newpacket = dhnpacket.dhnpacket(commands.Data(), localID, localID, settings.BackupInfoFileName(), Payload, supplierId)
                    transport_control.outboxAck(newpacket)
            reactor.callInThread(_send, Payload, self.localID)
        except:
            dhnio.Dprint(2, "backup_db.Save ERROR unable to save new XML file")
            dhnio.DprintException()

    def Load(self):
        #if os.path.exists(self._oldBackupsDir):
        #    self._ConvertOld()
        #    self.Save()
        #    os.rename(self._oldBackupsDir,self._oldBackupsDir+"_old")
        #elif
        if os.path.exists(self._xmlFile):
            self._loading = True
            self._LoadXML()
            self._loading = False
        #print self.ToXML()
        #self.Save()

    # used during developement to debug some stuff
    def CommandLog(self, command):
        dhnio.Dprint(6, 'backup_db: ' + str(command))
        #f = open("backup_commands.log", "a")
        #f.write(str(time.time()) + " " + command+"\n")
        #f.close()

    # begin of signing/encryption section, not yet active
    def SaveAddon(self):
        SessionKey = dhncrypto.NewSessionKey()
#        SessionKeyType = dhncrypto.SessionKeyType()
#        LocalID = misc.getLocalIdentity().getIDURL()
        Data = dhnio.ReadBinaryFile(self._xmlFile)
        DataLonger = misc.RoundupString(Data,24)
        self.EncryptedData = dhncrypto.EncryptWithSessionKey(SessionKey, DataLonger)
        self.Signature = self.GenerateSignature()  # usually just done at packet creation

    def SessionKey(self):
        return dhncrypto.DecryptLocalPK(self.EncryptedSessionKey)

    def GenerateHashBase(self):
        sep = "::::"
        # PREPRO needs to have all fields and separator
        StringToHash = self.CreatorID + sep + self.SessionKeyType + sep + self.EncryptedSessionKey + sep + self.Length + sep + self.EncryptedData
        return StringToHash

    def GenerateHash(self):
        return dhncrypto.Hash(self.GenerateHashBase())

    def GenerateSignature(self):
        return dhncrypto.Sign(self.GenerateHash())
## end of signing/encryption section, not yet active

    def AddDirectory(self, dirName, recursive=True, initialLoad=False):
        self.CommandLog("AddDirectory [%s] %s %s" % (dirName, str(recursive), str(initialLoad)))
        if not self.backupDirs.has_key(dirName):
            self.backupDirs[dirName] = _BackupDirectory(dirName, recursive)
        if not initialLoad:
            self.Save()

    def DeleteDirectory(self, dirName):
        #dhnio.Dprint(8, 'backup_db.DeleteDirectory ' + dirName)
        self.CommandLog("DelDirectory " + dirName)
        for dbDir in self.backupDirs.keys():
            if os.path.abspath(dirName) == os.path.abspath(dbDir):
            #if self.backupDirs.has_key(dirName):
                for backupRun in self.backupDirs[dbDir].backupRuns:
                    if backupRun.backupID not in self.deletedBackups:
                        self.deletedBackups.append(backupRun.backupID)
                del self.backupDirs[dbDir]
        self.Save()

    def CheckDirectory(self, dirName):
        #self.CommandLog("CheckDirectory " + dirName)
        if self.backupDirs.has_key(dirName):
            return True
        else:
            return False

    def GetBackupDirectories(self):
        #self.CommandLog("Get backup directories")
        backupDirectories = self.backupDirs.keys()
        backupDirectories.sort()
        return backupDirectories

    def GetDirectoryInfo(self, dirName):
        #self.CommandLog("GetDirectoryInfo " + dirName)
        if self.backupDirs.has_key(dirName):
            return self.backupDirs[dirName].GetDirectoryInfo()
        return '', 0, ''

    def AddDirBackup(self, dirName, backupID, backupStatus="", backupSize=0, backupStart=0, backupFinish=0, initialLoad=False):
        self.CommandLog("AddDirBackup [%s] %s %s %d %d %d %s" % (dirName, backupID, backupStatus, backupSize, backupStart, backupFinish, str(initialLoad)))
        if not self.backupDirs.has_key(dirName):
            self.backupDirs[dirName] = _BackupDirectory(dirName)
        self.backupDirs[dirName].AddBackupRun(backupID, backupSize, backupStatus, backupStart, backupFinish, initialLoad)
        if not initialLoad:
            self.Save()

    def DeleteDirBackup(self, backupID):
        self.CommandLog("DeleteDirBackup " + backupID)
        dirName = self.GetDirectoryFromBackupId(backupID)
        if self.backupDirs.has_key(dirName):
            self.backupDirs[dirName].DeleteDirBackup(backupID)
        if backupID not in self.deletedBackups:
            self.deletedBackups.append(backupID)
        self.Save()

    def IsDeleted(self, backupID):
        #self.CommandLog("IsDeleted " + backupID)
        if backupID in self.deletedBackups:
            return True
        else:
            return False

    def GetDirLastBackup(self, dirName):
        #self.CommandLog("GetDirLastBackup " + dirName)
        if self.backupDirs.has_key(dirName):
            if len(self.backupDirs[dirName].backupRuns) > 0:
                lastBackup = self.backupDirs[dirName].backupRuns[-1]
                return lastBackup.backupID, lastBackup.backupSize, lastBackup.backupStatus, lastBackup.backupStart, lastBackup.backupFinish
        return "", 0, "", 0, 0

    def GetDirBackupIds(self, dirName):
        if self.backupDirs.has_key(dirName):
            return self.backupDirs[dirName].GetDirBackupIds()
        else:
            return []

    def SetBackupStatus(self, dirName, backupID, backupStatus, backupFinish):
        self.CommandLog("SetBackupStatus " + dirName + " " + backupID + " " + backupStatus + " " + backupFinish)
        if self.backupDirs.has_key(dirName):
            self.backupDirs[dirName].SetBackupStatus(backupID, backupStatus, backupFinish)
            self.Save()

    def AbortDirectoryBackup(self, dirName):
        if self.backupDirs.has_key(dirName):
            self.backupDirs[dirName].runningBackup = False

    def GetLastRunInfo(self, dirName):
        #self.CommandLog("GetLastRunInfo " + dirName)
        return self.backupDirs[dirName].GetLastRunInfo()

    def IsBackupRunning(self, dirName):
        #self.CommandLog("GetDirectoryInfo " + dirName)
        if self.backupDirs.has_key(dirName):
            return self.backupDirs[dirName].IsBackupRunning()
        return False

    #def SetDirSchedule(self, dirName, scheduleType, scheduleTime, scheduleInverval, intervalDetails):
    def SetDirSchedule(self, dirName, sched):
        if self.backupDirs.has_key(dirName):
            self.CommandLog("SetDirSchedule %s %s %s %s %s" % (dirName, sched.type, sched.daytime, sched.interval, sched.details))
            self.backupDirs[dirName].SetDirSchedule(sched)
            self.Save()

    def GetDirSchedule(self, dirName):
        #self.CommandLog("GetDirSchedule" + dirName)
        if self.backupDirs.has_key(dirName):
            if self.backupDirs[dirName].schedule != None:
#                return self.backupDirs[dirName].schedule.scheduleType, self.backupDirs[dirName].schedule.scheduleTime, \
#                        self.backupDirs[dirName].schedule.scheduleInterval, self.backupDirs[dirName].schedule.intervalDetailsk
                return self.backupDirs[dirName].schedule.schedule
        return None

    def GetDirectorySubfoldersInclude(self, dirName):
        if self.backupDirs.has_key(dirName):
            return self.backupDirs[dirName].GetDirectorySubfoldersInclude()
        return True

    def SetDirectorySubfoldersInclude(self, dirName, backupSubfolders):
        if self.backupDirs.has_key(dirName):
            self.backupDirs[dirName].SetDirectorySubfoldersInclude(backupSubfolders)
        if not self._loading:
            self.Save()

    def GetDirectoryFromBackupId(self, backupID):
        #self.CommandLog("GetDirectoryFromBackupId " + backupID)
        for dirName in self.backupDirs.keys():
            for backupRun in self.backupDirs[dirName].backupRuns:
                if backupRun.backupID == backupID:
                    return dirName
        return ''

    def GetBackupIdRunInfo(self, backupID):
        for dirName in self.backupDirs.keys():
            for backupRun in self.backupDirs[dirName].backupRuns:
                if backupRun.backupID == backupID:
                    return backupRun.GetRunInfo()
        return None
    
    def GetBackupsByDateTime(self, reverse=False):
        d = {}
        for dirName in self.backupDirs.keys():
            for backupRun in self.backupDirs[dirName].backupRuns:
                d[backupRun.backupID] = (backupRun.backupID, dirName, backupRun.backupSize, backupRun.backupStatus)
        l = []
        for backupID in misc.sorted_backup_ids(d.keys(), reverse):
            l.append(d[backupID])
        d.clear()
        return l
    
    def GetBackupsByFolder(self):
        result = []
        for dirName in self.backupDirs.keys():
            dirBackupsSize = 0
            backupRuns = []
            for backupRun in self.backupDirs[dirName].backupRuns:
                backupRuns.append(backupRun)
                dirBackupsSize += backupRun.backupSize
            recentBackupID = '' if len(backupRuns) == 0 else backupRuns[0].backupID 
            result.append((dirName, backupRuns, dirBackupsSize, self.backupDirs[dirName].runningBackup, recentBackupID))
        return result 

    def Show(self):
        dbList = ""
        for dirName in self.backupDirs:
            dbList = dbList + self.backupDirs[dirName].Show()
        return dbList

    def GetBackupIds(self, full_info_please=False):
        backupIds = []
        for dirName in self.backupDirs:
            for backupRun in self.backupDirs[dirName].backupRuns:
                if full_info_please:
                    backupIds.append((str(backupRun.backupID), dirName, backupRun.backupSize, backupRun.backupStatus))
                else:
                    backupIds.append(str(backupRun.backupID))
        return misc.sorted_backup_ids(backupIds)

    def GetTotalBackupsSize(self):
        #self.CommandLog("GetTotalBackupsSize")
        backupsSize = 0
        for dirName in self.backupDirs:
            for backupRun in self.backupDirs[dirName].backupRuns:
                backupsSize += int(backupRun.backupSize)
        return backupsSize

    def ToXML(self):
        self.CommandLog("ToXML")
        dbXML = ''
##        dbXML += '<?xml version="1.0" encoding="ISO-8859-1"?>' + "\n"
        dbXML += '<?xml version="1.0" encoding="%s" ?>\n' % locale.getpreferredencoding()
        dbXML += "<backupData>\n"

        dbXML += "<backupDirectories>\n"
        for dirName in self.backupDirs:
            dbXML += self.backupDirs[dirName].ToXML()
        dbXML += "</backupDirectories>\n"

        dbXML += "<deletedBackups>\n"
        for backupID in self.deletedBackups[0-self._maxDeleted:]:
            dbXML += "  <deletedBackup>\n"
            dbXML += "    <deletedBackupId>" + backupID + "</deletedBackupId>\n"
            dbXML += "  </deletedBackup>\n"
        dbXML += "</deletedBackups>\n"

        dbXML += "</backupData>\n"
##        print 'ToXML', type(dbXML)
        return dbXML


    # the backup objects are backup.backup's
    def AddRunningBackupObject(self, backupID, backupObject):
        if not self.currentlyRunningBackups.has_key(backupID):
            self.currentlyRunningBackups[backupID] = backupObject
            
    def GetRunningBackupObject(self, backupID):
        return self.currentlyRunningBackups.get(backupID, None)

    def RemoveRunningBackupObject(self, backupID):
        if self.currentlyRunningBackups.has_key(backupID):
            del self.currentlyRunningBackups[backupID]

    def AbortRunningBackup(self, backupID):
        if self.currentlyRunningBackups.has_key(backupID):
            self.currentlyRunningBackups[backupID].abort()
        
    def RemoveAllRunningBackupObjects(self):
        self.currentlyRunningBackups.clear()

    def ShowRunningBackups(self):
        return self.currentlyRunningBackups.keys()

    def HasRunningBackup(self):
        if len(self.currentlyRunningBackups.keys()) > 0:
            return True
        return False
    
#------------------------------------------------------------------------------ 


InitDone = False

# We're keeping track of what directories we're backing up, what backups are
# associated with what directories
BackupDirs = None
AddDirectory = None                  # directory name
DeleteDirectory = None               # directory name
CheckDirectory = None                # directory name, return True if is in the list of directories we back up
GetBackupDirectories = None          # no input, return List of directory names
GetDirectoryInfo = None              # directory name - return
AddDirBackup = None                  # directory name, backup id, status, size, start, end
DeleteDirBackup = None               # backup id
IsDeleted = None                     # backup_id - is it in a list of deleted backup ids
SetBackupStatus = None               # directory name, backup id, status, finish time
AbortDirectoryBackup = None          # directory name - set running to false
GetLastRunInfo = None                # directory name - return last run info so backupID, backupSize, backupStatus, backupStart, backupFinish
IsBackupRunning = None               # directory name - return if an active backup is running
GetSchedule = None                   # directory name - return schedule type, time, interval, details
SetSchedule = None                   # directory name, schedule type, schedule time, schedule interval, interval details
GetLastRun = None                    # directory name - return last backup id, size, status, start, finish
GetDirBackupIds = None               # directory name - return list of all backup ids for the directory
GetBackupIds = None                  # list of all backup ids for all directories we're backing up
GetTotalBackupsSize = None           # adding up the backup runs to our best knowledge (will be a bit larger on suppliers?)
GetDirectoryFromBackupId = None      # backup id - return directory name
GetDirectorySubfoldersInclude = None # directory name - return if we back up subfolders
SetDirectorySubfoldersInclude = None # directory name, boolean (backup subfolders?)
GetBackupIdRunInfo = None            # Added by Veselin. Return RunInfo for a single BackupID
GetBackupsByDateTime = None          # return sorted by backupID list of tupples (backupID, backupDir, size, status)
GetBackupsByFolder = None            # return list of tupples (backupDir, backupRuns, totalBackupsSizeForDir, is_running, recentBackupID)   

AddRunningBackupObject = None        # backup id, running backup
GetRunningBackupObject = None        # backup id - return the backup running backup object
RemoveRunningBackupObject = None     # backup id
RemoveAllRunningBackupObjects = None # no arg, stop all running backups
HasRunningBackup = None
ShowRunningBackups = None
AbortRunningBackup = None

ToXML = None


def init():
    dhnio.Dprint(4,"backup_db.init")
    _local_backup_db = _BackupDB()
    global InitDone
    global BackupDirs
    global AddDirectory
    global DeleteDirectory
    global CheckDirectory
    global GetBackupDirectories
    global GetDirectoryInfo
    global AddDirBackup
    global DeleteDirBackup
    global IsDeleted
    global SetBackupStatus
    global AbortDirectoryBackup
    global GetLastRunInfo
    global IsBackupRunning
    global GetSchedule
    global SetSchedule
    global GetLastRun
    global GetDirBackupIds
    global GetBackupIds
    global GetTotalBackupsSize
    global GetDirectoryFromBackupId
    global GetDirectorySubfoldersInclude
    global SetDirectorySubfoldersInclude
    global GetBackupIdRunInfo
    global GetBackupsByDateTime
    global GetBackupsByFolder

    global AddRunningBackupObject
    global GetRunningBackupObject
    global RemoveRunningBackupObject
    global RemoveAllRunningBackupObjects
    global HasRunningBackup
    global ShowRunningBackups
    global AbortRunningBackup

    global ToXML

    BackupDirs = _local_backup_db.BackupDirs
    AddDirectory = _local_backup_db.AddDirectory
    DeleteDirectory = _local_backup_db.DeleteDirectory
    CheckDirectory = _local_backup_db.CheckDirectory
    GetBackupDirectories = _local_backup_db.GetBackupDirectories
    GetDirectoryInfo = _local_backup_db.GetDirectoryInfo
    AddDirBackup = _local_backup_db.AddDirBackup
    DeleteDirBackup = _local_backup_db.DeleteDirBackup
    IsDeleted = _local_backup_db.IsDeleted
    SetBackupStatus = _local_backup_db.SetBackupStatus
    AbortDirectoryBackup = _local_backup_db.AbortDirectoryBackup
    GetLastRunInfo = _local_backup_db.GetLastRunInfo
    IsBackupRunning = _local_backup_db.IsBackupRunning
    GetSchedule = _local_backup_db.GetDirSchedule
    SetSchedule = _local_backup_db.SetDirSchedule
    GetLastRun = _local_backup_db.GetDirLastBackup
    GetDirBackupIds = _local_backup_db.GetDirBackupIds
    GetBackupIds = _local_backup_db.GetBackupIds
    GetTotalBackupsSize = _local_backup_db.GetTotalBackupsSize
    GetDirectoryFromBackupId = _local_backup_db.GetDirectoryFromBackupId
    GetDirectorySubfoldersInclude = _local_backup_db.GetDirectorySubfoldersInclude
    SetDirectorySubfoldersInclude = _local_backup_db.SetDirectorySubfoldersInclude
    GetBackupIdRunInfo = _local_backup_db.GetBackupIdRunInfo
    GetBackupsByDateTime = _local_backup_db.GetBackupsByDateTime
    GetBackupsByFolder = _local_backup_db.GetBackupsByFolder

    AddRunningBackupObject = _local_backup_db.AddRunningBackupObject
    GetRunningBackupObject = _local_backup_db.GetRunningBackupObject
    RemoveRunningBackupObject = _local_backup_db.RemoveRunningBackupObject
    RemoveAllRunningBackupObjects = _local_backup_db.RemoveAllRunningBackupObjects
    HasRunningBackup = _local_backup_db.HasRunningBackup
    ShowRunningBackups = _local_backup_db.ShowRunningBackups
    AbortRunningBackup = _local_backup_db.AbortRunningBackup

    ToXML = _local_backup_db.ToXML
    
    _local_backup_db.Load()
    
    _local_backup_db.AskSizeForAllDirs()

    #init state
    InitDone = True


if __name__ == "__main__":
    init()
    import pprint
    pprint.pprint(GetBackupIds())
    pprint.pprint(GetBackupDirectories())
#    newdir = unicode(sys.argv[1])
#    print 'want to add:', newdir
##    AddDirBackup(newdir, "F3000000000AM", "done", "12344321", "1234554321.11", "1234554323.11")
#    AddDirectory(newdir)
##    DeleteDirectory(newdir)
#    print ToXML()
##    print GetDirectoryFromBackupId("F3000000000AM")
##    print CheckDirectory(newdir)
    #_local_backup_db.Save()



#!/usr/bin/python
#dhnnet.py
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#    some network routines
#    do not want to import any DHN modules here
# 

import os
import sys
import types
import socket
import urllib
import urllib2
import mimetypes

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in dhnnet.py')

from twisted.internet.defer import Deferred, succeed, fail
from twisted.web import iweb
from twisted.web import client
from twisted.web import http_headers
from twisted.web.client import _parse
from twisted.web.client import getPage
from twisted.web.client import downloadPage
from twisted.web.client import HTTPClientFactory
from twisted.web.client import HTTPDownloader
from twisted.internet import ssl
from twisted.internet import protocol
from twisted.protocols import basic

from zope.interface import implements

from OpenSSL import SSL

#------------------------------------------------------------------------------ 

_UserAgentString = "DataHaven.NET-http-agent"
_ProxySettings = {
    'host': '',
    'port': '',
    'ssl': 'False',
    'username':'',
    'password': ''
    }

#------------------------------------------------------------------------------

def init():
    pass

def detect_proxy_settings():
    d = {
        'host':'',
        'port':'',
        'username':'',
        'password':'',
        'ssl': 'False'}
    httpproxy = urllib2.getproxies().get('http', None)
    httpsproxy = urllib2.getproxies().get('https', None)

    if httpproxy is not None:
        try:
            scheme, host, port, path = _parse(httpproxy)
        except:
            return d
        d['ssl'] = 'False'
        d['host'] = host
        d['port'] = port

    if httpsproxy is not None:
        try:
            scheme, host, port, path = _parse(httpsproxy)
        except:
            return d
        d['ssl'] = 'True'
        d['host'] = host
        d['port'] = port

    return d

def set_proxy_settings(settings_dict):
    global _ProxySettings
    _ProxySettings = settings_dict

def get_proxy_settings():
    global _ProxySettings
    return _ProxySettings

def get_proxy_host():
    global _ProxySettings
    return _ProxySettings.get('host', '')

def get_proxy_port():
    global _ProxySettings
    try:
        return int(_ProxySettings.get('port', '8080'))
    except:
        return 8080

def get_proxy_username():
    global _ProxySettings
    return _ProxySettings.get('username', '')

def get_proxy_password():
    global _ProxySettings
    return _ProxySettings.get('password', '')

def get_proxy_ssl():
    global _ProxySettings
    return _ProxySettings.get('ssl', '')

def proxy_is_on():
    return get_proxy_host() != ''

#-------------------------------------------------------------------------------

class MyHTTPClientFactory(HTTPClientFactory):
    def page(self, page):
        if self.waiting:
            self.waiting = 0
            self.deferred.callback(page)

    def noPage(self, reason):
        if self.waiting:
            self.waiting = 0
            self.deferred.errback(reason)

    def clientConnectionFailed(self, _, reason):
        if self.waiting:
            self.waiting = 0
            self.deferred.errback(reason)


def getPageTwistedOld(url):
    global _UserAgentString
    scheme, host, port, path = _parse(url)
    factory = MyHTTPClientFactory(url, agent=_UserAgentString)
    reactor.connectTCP(host, port, factory)
    return factory.deferred

#------------------------------------------------------------------------------


def downloadPageTwisted(url, filename):
    global _UserAgentString
    return downloadPage(url, filename, agent=_UserAgentString)

#-------------------------------------------------------------------------------

#http://schwerkraft.elitedvb.net/plugins/scmcvs/cvsweb.php/enigma2-plugins/mediadownloader/src/HTTPProgressDownloader.py?rev=1.1;cvsroot=enigma2-plugins;only_with_tag=HEAD
class HTTPProgressDownloader(HTTPDownloader):
    """Download to a file and keep track of progress."""

    def __init__(self, url, fileOrName, writeProgress = None, *args, **kwargs):
        HTTPDownloader.__init__(self, url, fileOrName, supportPartial=0, *args, **kwargs)
        # Save callback(s) locally
        if writeProgress and type(writeProgress) is not list:
            writeProgress = [ writeProgress ]
        self.writeProgress = writeProgress

        # Initialize
        self.currentlength = 0
        self.totallength = None

    def gotHeaders(self, headers):
        HTTPDownloader.gotHeaders(self, headers)

        # If we have a callback and 'OK' from Server try to get length
        if self.writeProgress and self.status == '200':
            if headers.has_key('content-length'):
                self.totallength = int(headers['content-length'][0])
                for cb in self.writeProgress:
                    if cb:
                        cb(0, self.totallength)


    def pagePart(self, data):
        HTTPDownloader.pagePart(self, data)

        # If we have a callback and 'OK' from server increment pos
        if self.writeProgress and self.status == '200':
            self.currentlength += len(data)
            for cb in self.writeProgress:
                if cb:
                    cb(self.currentlength, self.totallength)




def downloadWithProgressTwisted(url, file, progress_func):
    global _UserAgentString
    scheme, host, port, path = _parse(url)
    factory = HTTPProgressDownloader(url, file, progress_func, agent=_UserAgentString)
    if scheme == 'https':
        contextFactory = ssl.ClientContextFactory()
        reactor.connectSSL(host, port, factory, contextFactory)
    else:
        reactor.connectTCP(host, port, factory)
    return factory.deferred

#-------------------------------------------------------------------------------


def downloadSSLWithProgressTwisted(url, file, progress_func, privateKeyFileName, certificateFileName):
    global _UserAgentString
    scheme, host, port, path = _parse(url)
    factory = HTTPProgressDownloader(url, file, progress_func, agent=_UserAgentString)
    if scheme != 'https':
        return None
    contextFactory = ssl.DefaultOpenSSLContextFactory(privateKeyFileName, certificateFileName)
    reactor.connectSSL(host, port, factory, contextFactory)
    return factory.deferred


#-------------------------------------------------------------------------------


class MyClientContextFactory(ssl.ClientContextFactory):
    def __init__(self, certificates_filenames):
        self.certificates_filenames = list(certificates_filenames)

    def verify(self, connection, x509, errnum, errdepth, ok):
        return ok

    def getContext(self):
        ctx = ssl.ClientContextFactory.getContext(self)
        for cert in self.certificates_filenames:
            try:
                ctx.load_verify_locations(cert)
            except:
                pass
        ctx.set_verify(SSL.VERIFY_PEER | SSL.VERIFY_FAIL_IF_NO_PEER_CERT , self.verify)
        return ctx


def downloadSSL(url, fileOrName, progress_func, certificates_filenames):
    global _UserAgentString
    scheme, host, port, path = _parse(url)
    if scheme != 'https':
        return fail('expect only https protocol')
    if not isinstance(certificates_filenames, types.ListType):
        certificates_filenames = [certificates_filenames, ]
    cert_found = False
    for cert in certificates_filenames:
        if os.path.isfile(cert) and os.access(cert, os.R_OK):
            cert_found = True
            break
    if not cert_found:
        return fail('no one ssl certificate found')
    factory = HTTPDownloader(url, fileOrName, agent=_UserAgentString)
    contextFactory = MyClientContextFactory(certificates_filenames)
    reactor.connectSSL(host, port, factory, contextFactory)
    return factory.deferred

#------------------------------------------------------------------------------

class NoVerifyClientContextFactory:
    isClient = 1
    method = SSL.SSLv3_METHOD
    def getContext(self):
        def x(*args):
            return True
        ctx = SSL.Context(self.method)
        #print dir(ctx)
        ctx.set_verify(SSL.VERIFY_NONE,x)
        return ctx

def downloadSSL2(url, fileOrName, progress_func, certificates_filenames=[]):
    global _UserAgentString
    factory = HTTPDownloader(url, fileOrName, agent=_UserAgentString)

    if proxy_is_on():
        factory.path = url
        # TODO
        # need to check certificate too
        contextFactory = MyClientContextFactory(certificates_filenames)
        reactor.connectSSL(get_proxy_host(), get_proxy_port(), factory, contextFactory)

    else:
        scheme, host, port, path = _parse(url)
        if not isinstance(certificates_filenames, types.ListType):
            certificates_filenames = [certificates_filenames, ]
        cert_found = False
        for cert in certificates_filenames:
            if os.path.isfile(cert) and os.access(cert, os.R_OK):
                cert_found = True
                break
        if not cert_found:
            return fail('no one ssl certificate found')
        contextFactory = MyClientContextFactory(certificates_filenames)
        reactor.connectSSL(host, port, factory, contextFactory)

    return factory.deferred

#-------------------------------------------------------------------------------

class ProxyClientFactory(client.HTTPClientFactory):
    def setURL(self, url):
        client.HTTPClientFactory.setURL(self, url)
        self.path = url

def getPageTwisted(url):
    global _UserAgentString
    if proxy_is_on():
        factory = ProxyClientFactory(url, agent=_UserAgentString)
        reactor.connectTCP(get_proxy_host(), get_proxy_port(), factory)
        return factory.deferred
    else:
        return getPage(url, agent=_UserAgentString)

#------------------------------------------------------------------------------

def downloadHTTP(url, fileOrName):
    global _UserAgentString
    scheme, host, port, path = _parse(url)
    factory = HTTPDownloader(url, fileOrName, agent=_UserAgentString)
    if proxy_is_on():
        host = get_proxy_host()
        port = get_proxy_port()
        factory.path = url
    reactor.connectTCP(host, port, factory)
    return factory.deferred

#-------------------------------------------------------------------------------

def IpIsLocal(ip):
    if ip == '':
        return True
    if ip == '0.0.0.0':
        return True
    if ip.startswith('192.168.'):
        return True
    if ip.startswith('10.'):
        return True
    if ip.startswith('127.'):
        return True
    if ip.startswith('172.'):
        try:
            secondByte = int(ip.split('.')[1])
        except:
            raise Exception('wrong ip address ' + str(ip))
            return True
        if secondByte >= 16 and secondByte <= 31:
            return True
    return False

#-------------------------------------------------------------------------------

def getLocalIpError():
    import socket
    addr = socket.gethostbyname(socket.gethostname())
    if addr == "127.0.0.1" and os.name == 'posix':
        import commands
        output = commands.getoutput("/sbin/ifconfig")
        #TODO parseaddress not done yet
        addr = parseaddress(output)
    return addr


#http://ubuntuforums.org/showthread.php?t=1215042
def getLocalIp(): # had this in p2p/stun.py
    import socket
    # 1: Use the gethostname method

    try:
        ipaddr = socket.gethostbyname(socket.gethostname())
        if not( ipaddr.startswith('127') ) :
            #print('Can use Method 1: ' + ipaddr)
            return ipaddr
    except:
        pass

    # 2: Use outside connection
    '''
    Source:
    http://commandline.org.uk/python/how-to-find-out-ip-address-in-python/
    '''

    ipaddr=''
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        s.connect(('google.com', 0))
        ipaddr=s.getsockname()[0]
        #print('Can used Method 2: ' + ipaddr)
        return ipaddr
    except:
        pass


    # 3: Use OS specific command
    import subprocess , platform
    ipaddr=''
    os_str=platform.system().upper()

    if os_str=='LINUX' :

        # Linux:
        arg='ip route list'
        p=subprocess.Popen(arg,shell=True,stdout=subprocess.PIPE)
        data = p.communicate()
        sdata = data[0].split()
        ipaddr = sdata[ sdata.index('src')+1 ]
        #netdev = sdata[ sdata.index('dev')+1 ]
        #print('Can used Method 3: ' + ipaddr)
        return ipaddr

    elif os_str=='WINDOWS' :

        # Windows:
        arg='route print 0.0.0.0'
        p=subprocess.Popen(arg,shell=True,stdout=subprocess.PIPE)
        data = p.communicate()
        strdata=data[0].decode()
        sdata = strdata.split()

        while len(sdata)>0:
            if sdata.pop(0)=='Netmask' :
                if sdata[0]=='Gateway' and sdata[1]=='Interface' :
                    ipaddr=sdata[6]
                    break
        #print('Can used Method 4: ' + ipaddr)
        return ipaddr

    return '127.0.0.1' # uh oh, we're in trouble, but don't want to return none

#-------------------------------------------------------------------------------

def TestInternetConnection(remote_host='www.google.com'):
    try:
        (family, socktype, proto, garbage, address) = socket.getaddrinfo(remote_host, "http")[0]
    except Exception, e:
        return False

    s = socket.socket(family, socktype, proto)

    try:
        result = s.connect(address)
    except Exception, e:
        return False

    return result is None or result == 0


#------------------------------------------------------------------------------


def SendEmail(TO, FROM, HOST, PORT, LOGIN, PASSWORD, SUBJECT, BODY, FILES):
#    try:
        import smtplib
        from email import Encoders
        from email.MIMEText import MIMEText
        from email.MIMEBase import MIMEBase
        from email.MIMEMultipart import MIMEMultipart
        from email.Utils import formatdate

        msg = MIMEMultipart()
        msg["From"] = FROM
        msg["To"] = TO
        msg["Subject"] = SUBJECT
        msg["Date"]    = formatdate(localtime=True)
        msg.attach(MIMEText(BODY))

        # attach a file
        for filePath in FILES:
            if not os.path.isfile(filePath):
                continue
            part = MIMEBase('application', "octet-stream")
            part.set_payload( open(filePath,"rb").read() )
            Encoders.encode_base64(part)
            part.add_header('Content-Disposition', 'attachment; filename="%s"' % os.path.basename(filePath))
            msg.attach(part)

        s = smtplib.SMTP(HOST, PORT)

        #s.set_debuglevel(True) # It's nice to see what's going on

        s.ehlo() # identify ourselves, prompting server for supported features

        if s.has_extn('STARTTLS'):
            s.starttls()
            s.ehlo() # re-identify ourse

        s.login(LOGIN, PASSWORD)  # optional

        failed = s.sendmail(FROM, TO, msg.as_string())

        s.close()

#    except:
#        dhnio.DprintException()


#-------------------------------------------------------------------------------

# Great Thanks to Mariano!
# http://marianoiglesias.com.ar/python/file-uploading-with-multi-part-encoding-using-twisted/

class StringReceiver(protocol.Protocol):
    buffer = ""

    def __init__(self, deferred=None):
        self._deferred = deferred

    def dataReceived(self, data):
        self.buffer += data

    def connectionLost(self, reason):
        if self._deferred and reason.check(client.ResponseDone):
            self._deferred.callback(self.buffer)
        else:
            self._deferred.errback(self.buffer)
            
class MultiPartProducer:
    implements(iweb.IBodyProducer)
    CHUNK_SIZE = 2 ** 8
    def __init__(self, files={}, data={}, callback=None, deferred=None):
        self._files = files
        self._file_lengths = {}
        self._data = data
        self._callback = callback
        self._deferred = deferred
        self.boundary = self._boundary()
        self.length = self._length()
        
    def startProducing(self, consumer):
        self._consumer = consumer
        self._current_deferred = Deferred()
        self._sent = 0
        self._paused = False
        if not hasattr(self, "_chunk_headers"):
            self._build_chunk_headers()
        if self._data:
            block = ""
            for field in self._data:
                block += self._chunk_headers[field]
                block += self._data[field]
                block += "\r\n"
            self._send_to_consumer(block)
        if self._files:
            self._files_iterator = self._files.iterkeys()
            self._files_sent = 0
            self._files_length = len(self._files)
            self._current_file_path = None
            self._current_file_handle = None
            self._current_file_length = None
            self._current_file_sent = 0
            result = self._produce()
            if result:
                return result
        else:
            return succeed(None)
        return self._current_deferred
    
    def resumeProducing(self):
        self._paused = False
        result = self._produce()
        if result:
            return result
        
    def pauseProducing(self):
        self._paused = True
        
    def stopProducing(self):
        self._finish(True)
        if self._deferred and self._sent < self.length:
            self._deferred.errback(Exception("Consumer asked to stop production of request body (%d sent out of %d)" % (self._sent, self.length)))
            
    def _produce(self):
        if self._paused:
            return
        done = False
        while not done and not self._paused:
            if not self._current_file_handle:
                field = self._files_iterator.next()
                self._current_file_path = self._files[field]
                self._current_file_sent = 0
                self._current_file_length = self._file_lengths[field]
                self._current_file_handle = open(self._current_file_path, "rb")
                self._send_to_consumer(self._chunk_headers[field])
            chunk = self._current_file_handle.read(self.CHUNK_SIZE)
            if chunk:
                self._send_to_consumer(chunk)
                self._current_file_sent += len(chunk)
            if not chunk or self._current_file_sent == self._current_file_length:
                self._send_to_consumer("\r\n")
                self._current_file_handle.close()
                self._current_file_handle = None
                self._current_file_sent = 0
                self._current_file_path = None
                self._files_sent += 1
            if self._files_sent == self._files_length:
                done = True
        if done:
            self._send_to_consumer("--%s--\r\n" % self.boundary)
            self._finish()
            return succeed(None)
        
    def _finish(self, forced=False):
        if hasattr(self, "_current_file_handle") and self._current_file_handle:
            self._current_file_handle.close()
        if self._current_deferred:
            self._current_deferred.callback(self._sent)
            self._current_deferred = None
        if not forced and self._deferred:
            self._deferred.callback(self._sent)
            
    def _send_to_consumer(self, block):
        self._consumer.write(block)
        self._sent += len(block)
        if self._callback:
            self._callback(self._sent, self.length)
            
    def _length(self):
        self._build_chunk_headers()
        length = 0
        if self._data:
            for field in self._data:
                length += len(self._chunk_headers[field])
                length += len(self._data[field])
                length += 2
        if self._files:
            for field in self._files:
                length += len(self._chunk_headers[field])
                length += self._file_size(field)
                length += 2
        length += len(self.boundary)
        length += 6
        return length
    
    def _build_chunk_headers(self):
        if hasattr(self, "_chunk_headers") and self._chunk_headers:
            return
        self._chunk_headers = {}
        for field in self._files:
            self._chunk_headers[field] = self._headers(field, True)
        for field in self._data:
            self._chunk_headers[field] = self._headers(field)
            
    def _headers(self, name, is_file=False):
        value = self._files[name] if is_file else self._data[name]
        _boundary = self.boundary.encode("utf-8") if isinstance(self.boundary, unicode) else urllib.quote_plus(self.boundary)
        headers = ["--%s" % _boundary]
        if is_file:
            disposition = 'form-data; name="%s"; filename="%s"' % (name, os.path.basename(value))
        else:
            disposition = 'form-data; name="%s"' % name
        headers.append("Content-Disposition: %s" % disposition)
        if is_file:
            file_type = self._file_type(name)
        else:
            file_type = "text/plain; charset=utf-8"
        headers.append("Content-Type: %s" % file_type)
        if is_file:
            headers.append("Content-Length: %i" % self._file_size(name))
        else:
            headers.append("Content-Length: %i" % len(value))
        headers.append("")
        headers.append("")
        return "\r\n".join(headers)
    
    def _boundary(self):
        boundary = None
        try:
            import uuid
            boundary = uuid.uuid4().hex
        except ImportError:
            import random, sha
            bits = random.getrandbits(160)
            boundary = sha.new(str(bits)).hexdigest()
        return boundary
    
    def _file_type(self, field):
        type = mimetypes.guess_type(self._files[field])[0]
        return type.encode("utf-8") if isinstance(type, unicode) else str(type)
    
    def _file_size(self, field):
        size = 0
        try:
            handle = open(self._files[field], "r")
            size = os.fstat(handle.fileno()).st_size
            handle.close()
        except:
            size = 0
        self._file_lengths[field] = size
        return self._file_lengths[field]
       
#------------------------------------------------------------------------------ 

def uploadHTTP(url, files, data, progress=None):
    producerDeferred = Deferred()
    receiverDeferred = Deferred()
    
    myProducer = MultiPartProducer(files, data, progress, producerDeferred)
    myReceiver = StringReceiver(receiverDeferred)
    
    headers = http_headers.Headers()
    headers.addRawHeader("Content-Type", "multipart/form-data; boundary=%s" % myProducer.boundary)
    
    agent = client.Agent(reactor)
    request = agent.request("POST", url, headers, myProducer)
    request.addCallback(lambda response: response.deliverBody(myReceiver))
    
    return receiverDeferred

#------------------------------------------------------------------------------ 


def test1():
    def done(x, filen):
        print 'done'
        print x
        print open(filen).read()
    def fail(x):
        print 'fail'
        print x.getErrorMessage()

    url = 'https://downloads.datahaven.net/info.txt'
    fn = '123'
    d = downloadSSL1(url, fn, None, 'dhn.cer')
    d.addCallback(done, fn)
    d.addErrback(fail)

def test2():
    def done(x):
        #print 'done!!!!!!!!!!!!!!!!!!!!!!!!!!!'
        print x
    def fail(x):
        print 'fail'
        print x
        print x.getErrorMessage()
    url = 'http://identity.datahaven.net/veselin.xml'
    d = getPageTwisted(url)
    d.addCallback(done)
    d.addErrback(fail)

def test3():
    d = detect_proxy_settings()
    print d

def test4():
    def done(x, filen):
        print 'done'
        print x
        print open(filen).read()
    def fail(x):
        print 'fail'
        print x.getErrorMessage()

#    import settings
#    url = settings.UpdateLocationURL() + settings.CurrentVersionDigestsFilename()
    url = 'http://identity.datahaven.net/downloads/version.txt'
    fn = '123'
    d = downloadHTTP(url, fn)
    d.addCallback(done, fn)
    d.addErrback(fail)

def test5():
    SendEmail('to.datahaven@gmail.com',
              'dhnemail1@mail.offshore.ai',
              'smtp.gmail.com',
              587,
              'datahaven.net.mail@gmail.com',
              'datahaven.net.mail',
              'subj',
              'some body \n\n body',
              ['__init__.pyc'],)



if __name__ == '__main__':
    #init()
    test5()
    #reactor.run()





#!/usr/bin/python
#dhnstarter.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2011
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#

import os
import sys
import hashlib
import time
import subprocess
import tempfile
import win32process

from distutils.dir_util import copy_tree
from twisted.python.win32 import cmdLineQuote

import EasyDialogs

#------------------------------------------------------------------------------

def sharedPath(filename, subdir='logs'):
    sharedDir = os.path.join(
                        #os.path.expanduser('~'),
                        #'Application Data',
                        os.environ['APPDATA'],
                        'DataHaven.NET',
                        subdir)
    if filename is None:
        return os.path.abspath(sharedDir)
    return os.path.abspath(os.path.join(sharedDir, filename))

#------------------------------------------------------------------------------ 

StarterFilename = os.path.basename(sys.argv[0].lower()) #'dhnstarter.exe'

UpdateRepo = ''
UpdateLocationURL = ''
#DefaultRepo = 'test'
#DefaultUpdateLocationURL = 'http://identity.datahaven.net/downloads/'
DefaultRepo = 'devel'
DefaultUpdateLocationURL = 'http://datahaven.net/repo/devel/'

UpdateFolder = 'windows/'
FilesDigestsFilename = 'info.txt'
CurrentVersionDigestsFilename = 'version.txt'
WGetFilename = 'wget.exe'
MainExecutableFilename = 'dhnmain.exe'
StarterFilename = 'dhnstarter.exe'

LogFilePath = sharedPath('dhnstarter.log', 'logs')
RepoFileName = sharedPath('repo', 'metadata')
FilesDigestsLocalPath = sharedPath('info', 'metadata')
CurrentVersionDigestsLocalPath = sharedPath('version', 'metadata')
WGetLogFilename = sharedPath('wget.log', 'logs')

#ONLY LOWER CASE
ExcludesFiles = [
    'w9xpopen.exe',
    'MSVCR71.dll'.lower(),
    'python25.dll',
    WGetFilename,
    StarterFilename,
    CurrentVersionDigestsFilename,
    FilesDigestsFilename,
    ]

#------------------------------------------------------------------------------

def logcheck():
    global LogFilePath
    try:
        if os.path.getsize(LogFilePath) >= 100 * 1024:
            os.remove(LogFilePath)
    except:
        pass

def logwrite(txt, mode='a'):
    global LogFilePath
    fout = open(LogFilePath, mode)
    fout.write(txt)
    fout.close()

def read_file(filename, mode='rb'):
    try:
        file = open(filename, mode)
        data = file.read()
        file.close()
        if mode == 'r':
            data = data.replace('\r\n','\n')
        return data
    except:
        return ''

def make_hash(data):
    return hashlib.md5(data).hexdigest()

def file_hash(filename):
    return make_hash(read_file(filename))

def run_wget(url, filename):
    cmd = '%s --no-check-certificate -o%s -O%s -T60 %s' % (WGetFilename, cmdLineQuote(WGetLogFilename), cmdLineQuote(filename), url)
    logwrite('%s\n' % cmd)
    return subprocess.call(cmd, shell=True)

def find_process_win32(applist):
    pidsL = []
    try:
        import win32com.client
        objWMI = win32com.client.GetObject("winmgmts:\\\\.\\root\\CIMV2")
        colProcs = objWMI.ExecQuery("SELECT * FROM Win32_Process")
        for Item in colProcs:
            pid = int(Item.ProcessId)
            if pid == os.getpid():
                continue
            search_str = Item.Caption.lower()
            if Item.CommandLine:
                search_str += Item.CommandLine.lower()
            for app in applist:
                if search_str.find(app) != -1:
                    pidsL.append(pid)
    except:
        logwrite('error in find_process_win32\n')
    return pidsL

def kill_process_win32(pid):
    try:
        from win32api import TerminateProcess, OpenProcess, CloseHandle
        PROCESS_TERMINATE = 1
        handle = OpenProcess(PROCESS_TERMINATE, False, pid)
        TerminateProcess(handle, -1)
        CloseHandle(handle)
    except:
        logwrite('error in kill_process_win32\n')

def stop_all(search_list):
    total_count = 0
    while True:
        count = 0
        appList = find_process_win32(search_list)
        for pid in appList:
            count +=1
            kill_process_win32(pid)
        if len(appList) == 0:
            break
        total_count += 1
        if total_count > 10:
            return 'error killing process '+str(appList)
        time.sleep(1)
    return 0

def launch(show):
    logwrite('launch %s show=%s\n' % (MainExecutableFilename, show))
    try:
        if show:
            subprocess.Popen([MainExecutableFilename, 'show'])
        else:
            subprocess.Popen([MainExecutableFilename,])
    except:
        return 'error starting dhnmain.exe'
    return 0

def uninstall():
    def make_bat_file():
        wait_appname = StarterFilename
        local_dir = os.path.abspath(os.path.dirname(os.path.abspath(sys.executable)))
        dirs2delete = [ sharedPath(None, '.'), os.path.join(tempfile.gettempdir(), 'dhn') ]
        logwrite('make_bat_file\n')
        batfileno, batfilename = tempfile.mkstemp('.bat', 'DataHaven.NET-uninstall-')
        logwrite('batfilename:%s\n' % batfilename)
        logwrite('local_dir:%s\n' % local_dir)
        logwrite('dirs2delete:%s\n' % '\n'.join(dirs2delete))
        batsrc = ''
        batsrc += 'cd "%s"\n' % local_dir
        batsrc += 'del search.log /Q\n'
        batsrc += ':again\n'
        batsrc += 'sleep 1\n'
        batsrc += 'tasklist /FI "IMAGENAME eq %s" /FO CSV > search.log\n' % wait_appname
        batsrc += 'FOR /F %%A IN (search.log) DO IF %%-zA EQU 0 GOTO again\n'
        batsrc += 'cd "%s"\n' % os.path.dirname(os.path.abspath(batfilename))
        for dirpath in dirs2delete:
            batsrc += 'rmdir /S /Q "%s"\n' % os.path.abspath(dirpath)
            one_copy = 'if exist "%s\\NUL" rmdir /S /Q "%s"\n' % (os.path.abspath(dirpath), os.path.abspath(dirpath))
            batsrc += (one_copy * 20) 
        batsrc += 'rem del /F /S /Q "%s"\n' % os.path.abspath(batfilename)
        os.write(batfileno, batsrc)
        os.close(batfileno)
        logwrite('bat source:\n')
        logwrite(batsrc)
        return os.path.abspath(batfilename)
    
    def remove_registry_uninstall():
        try:
            import _winreg
        except:
            return False
        unistallpath = "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall" 
        regpath = unistallpath + "\\DataHaven.NET"
    
        # open
        try:
            reg = _winreg.OpenKey(_winreg.HKEY_LOCAL_MACHINE, regpath, 0, _winreg.KEY_ALL_ACCESS)
        except:
            try:
                reg = _winreg.CreateKey(_winreg.HKEY_LOCAL_MACHINE, regpath)
            except:
                dhnio.DprintException()
                return False
    
        # check
        i = 0
        while True:
            try:
                name, value, typ = _winreg.EnumValue(reg, i)
            except:
                break
            i += 1
            try:
                _winreg.DeleteKey(reg, name)
            except:
                try:
                    _winreg.DeleteValue(reg, name)
                except:
                    pass
    
        # delete
        _winreg.CloseKey(reg)
        try:
            reg = _winreg.OpenKey(_winreg.HKEY_LOCAL_MACHINE, unistallpath, 0, _winreg.KEY_ALL_ACCESS)
        except:
            return False
        try:
            _winreg.DeleteKey(reg, 'DataHaven.NET')
        except:
            _winreg.CloseKey(reg)
            return False
        _winreg.CloseKey(reg)
        return True
    
    logwrite('uninstall()\n')
    ret = stop_all([
            'dhnstarter.exe',
            'dhnmain.exe',
            'dhnview.exe'
            'dhntester.exe',
            'dhnbackup.exe'])
    if ret != 0:
        return ret
    logwrite('start uninstalling\n')
    remove_registry_uninstall()
    batfilename = make_bat_file()
    #misc.UpdateRegistryUninstall(True)
    cmd = os.path.abspath(batfilename).replace('\\', '/')
    logwrite('cmd:%s\n' % cmd)
    p = subprocess.Popen(cmd, shell=True, creationflags=win32process.CREATE_NO_WINDOW,)
    return ret
       

def main():
    logwrite('main()\n')
    
    global ExcludesFiles
    global UpdateLocationURL
    global UpdateRepo
    global RepoFileName
    global DefaultUpdateLocationURL
    global DefaultRepo
    global FilesDigestsFilename
    global FilesDigestsLocalPath
    global CurrentVersionDigestsLocalPath
    global StarterFilename
    
    #if we need to stop - we kill all running instances and exit
    if sys.argv.count('stop'):
        #we want to check if another instance is running
        return stop_all([
            'dhnstarter.exe',
            'dhnmain.exe',
            'dhnmain.py',
            'dhnview.exe',
            'dhnview.py',
            'dhn.py',])

    search_list = ['dhnstarter.exe',]
    # if we do not need to stop,
    # but still other instance is running
    # we do not want to work together. EXIT.
    if len(find_process_win32(search_list)) > 0:
        logwrite('another dhnstarter.exe found working')
        return 0

    show = False
    if sys.argv.count('show') or sys.argv.count('install'):
        show = True

    if not show:
        #if dhnmain.exe or dhnview.exe is running - stop it.
        search_list.extend([
            'dhnmain.exe',
            'dhnmain.py',
            'dhnview.exe',
            'dhnview.py',
            'dhn.py',
            ])
        res = stop_all(search_list)
        if res != 0:
            return res

    #read file "info"
    if os.path.isfile(FilesDigestsLocalPath):
        src = read_file(FilesDigestsLocalPath, 'r')
        if src == '':
            logwrite('file %s is empty\n' % FilesDigestsLocalPath)
    else:
        logwrite('file not found: %s\n' % FilesDigestsLocalPath)
        src = read_file(os.path.join(os.path.dirname(
            os.path.abspath(sys.executable)), FilesDigestsFilename), 'r')
        if src != '':
            logwrite('take %s from the current executable folder\n' % FilesDigestsFilename)
        else:
            logwrite('file %s in the current executable folder is empty or not exist\n' % FilesDigestsFilename)
            
    logwrite('info length: ' + str(len(src)) + '\n')

    if os.path.isfile(CurrentVersionDigestsLocalPath):
        cur_version = read_file(CurrentVersionDigestsLocalPath, 'r').strip()
        if cur_version == '':
            logwrite('file %s is empty\n' % CurrentVersionDigestsLocalPath)
    else:
        logwrite('file not found: %s\n' % CurrentVersionDigestsLocalPath)
        cur_version = read_file(os.path.join(os.path.dirname(
            os.path.abspath(sys.executable)), CurrentVersionDigestsFilename), 'r').strip()
        if cur_version != '':
            logwrite('take %s from the current executable folder\n' % CurrentVersionDigestsFilename)
        else:
            logwrite('file %s in the current executable folder is empty or not exist\n' % CurrentVersionDigestsFilename)

    logwrite('current version is: %s\n' % cur_version)

    #if local info is not exist, or it is empty - we want to download it from the server
    if src.strip() == '':
        url = UpdateLocationURL + FilesDigestsFilename
        logwrite('want to download %s\n' % url)

        if run_wget(url, FilesDigestsLocalPath):
            logwrite('wget error url=%s\n' % url)
            return 1

        if not os.path.isfile(FilesDigestsLocalPath):
            logwrite('file not found %s\n' % FilesDigestsLocalPath)
            return 1

        src = read_file(FilesDigestsLocalPath, 'r')
        if src == '':
            logwrite('error when reading from file %s\n' % FilesDigestsLocalPath)
            return 1

    # make new version digest from info file
    new_version = make_hash(src)
    logwrite('the new version is: %s\n' % new_version)

    # and compare with current version
    if new_version == cur_version and cur_version != '':
        # digests are equal so our binaries have latest version - ready to start
        logwrite('version was not changed\n')

        # also try to remove info.txt and version.txt in the current folder
        # they can be here because included in the installation release
        # so they needed only during first start of the dhnstarter.exe
        # if we did not start with param "install" - we do not need them at all
        if sys.argv.count('install') == 0:
            try:
                if os.path.exists('info.txt'):
                    os.remove('info.txt')
                if os.path.exists('version.txt'):
                    os.remove('version.txt')
            except:
                logwrite('can not remove info.txt or version.txt from current folder\n')
        return launch(show)

    # so versions is not the same - save the new version 
    logwrite('save the new version to %s\n' % CurrentVersionDigestsLocalPath)
    try:
        fout = open(CurrentVersionDigestsLocalPath, 'w')
        fout.write(new_version)
        fout.close()
    except:
        logwrite('error writing to %s\n' % CurrentVersionDigestsLocalPath)

    #reading files hash values from info file
    current_info = {}
    for line in src.strip().split('\n'):
        words = line.split(' ')
        if len(words) < 2:
            continue
        fname = words[1].strip()
        #we do not want to check or download this files:
        if os.path.basename(fname.lower()) in ExcludesFiles:
            continue
        current_info[fname] = words[0].strip()

    logwrite('current info files ' + str(len(current_info)) +'\n')

    #read hash values for all files in the current folder
    local_info = {}
    logwrite('current path:  %s\n' % os.path.abspath('.'))
    # executable_filename = os.path.abspath(sys.argv[0])
    # fullpath = os.path.dirname(executable_filename)
    # fullpath = os.path.abspath('.')
    fullpath = sharedPath('', 'bin')
    logwrite('new path:      %s\n' % fullpath)
    for root, dirs, files in os.walk(fullpath):
        for fname in files:
            abspath = os.path.abspath(os.path.join(root, fname))
            relpath = abspath[len(fullpath)+1:]
            local_info[relpath] = file_hash(abspath)

    logwrite('local files %s\n' % str(len(local_info)) )

    #compare hash values
    download_list = []
    for fpath in current_info.keys():
        #if some file are missing - we want to download it
        if not local_info.has_key(fpath):
            logwrite('%s - missied\n' % fpath)
            download_list.append(fpath)
            continue
        #if some file hash is different - we want to download it and overwrite existing
        if local_info[fpath] != current_info[fpath]:
            logwrite('%s - hash is not the same\n' % fpath)
            download_list.append(fpath)
            continue

    logwrite('files to download: %s\n' % str(len(download_list)))

    download_list.sort()

    if len(download_list) > 0:
        try:
            dlg = None
            dlgshow = False

            if show:
                dlgshow = True

            if dlgshow:
                dlg = EasyDialogs.ProgressBar(
                    'Updating DataHaven.NET',
                    len(download_list),
                    '')

            #we want to download needed files
            for path in download_list:
                url = UpdateLocationURL + UpdateFolder + path
                url = url.replace('\\', '/')
                filename = os.path.join('.', path)
                dirname = os.path.dirname(filename)
##                if os.path.exists(filename):
##                    try:
##                        os.remove(filename)
##                    except:
##                        pass
                if not os.path.isdir(dirname):
                    try:
                        os.makedirs(dirname)
                    except:
                        pass

                if dlg:
                    dlg.label( '%s' % (path) )

                filenameWget = filename+'.wget'
                if os.path.isfile(filenameWget):
                    try:
                        os.remove(filenameWget)
                    except:
                        logwrite('wget warning can not remove filename=%s\n' % filenameWget)

                if run_wget(url, filenameWget):
                    logwrite('wget error url=%s filename=%s\n' % (url, filenameWget))

                else:
                    h = file_hash(filenameWget)
                    if h == current_info[path]:
                        try:
                            if os.path.isfile(filename):
                                os.remove(filename)
                                logwrite('REMOVED: %s\n' % path)
                        except:
                            logwrite('can not remove %s\n' % (filename))
                        try:
                            os.rename(filenameWget, filename)
                            logwrite('UPDATED: %s\n' % path)
                        except:
                            logwrite('can not rename %s->%s\n' % (filenameWget, filename))
                    else:
                        logwrite('incorrect file %s :  hash values is not equal\n' % path)

                if dlg:
                    dlg.inc()

            if dlg:
                del dlg
                dlg = None

        except KeyboardInterrupt:
            return 'canceled'

    #finally - start dhnmain.exe and exit
    return launch(show)


def run():
    global UpdateLocationURL
    global UpdateRepo
    global RepoFileName
    global DefaultUpdateLocationURL
    global DefaultRepo
    global FilesDigestsFilename
    global FilesDigestsLocalPath
    global CurrentVersionDigestsLocalPath
    global StarterFilename
    
    if hasattr(sys, 'frozen'):
        executable_filename = os.path.abspath(sys.executable)
    else:
        executable_filename = os.path.abspath(sys.argv[0])
    sharedStarterFilename = sharedPath(StarterFilename, 'bin')
    binDirExist = os.path.isdir(sharedPath(None, 'bin'))

    if not os.path.isdir(sharedPath('', 'bin')):
        os.makedirs(sharedPath('', 'bin'))

    if not os.path.isdir(sharedPath('', 'logs')):
        os.makedirs(sharedPath('', 'logs'))

    if not os.path.isdir(sharedPath('', 'metadata')):
        os.makedirs(sharedPath('', 'metadata'))

    logcheck()
    if sys.argv.count('uninstall'):
        return uninstall()

    try:
        UpdateRepo, UpdateLocationURL = read_file(RepoFileName, 'r').split('\n')
    except:
        logwrite('can\'t read repo file from %s\n' % RepoFileName)
        UpdateRepo = ''
        UpdateLocationURL = ''
    if UpdateRepo.strip() == '' or UpdateLocationURL.strip() == '':
        logwrite('empty repo file, use default repo\n')
        UpdateRepo = DefaultRepo
        UpdateLocationURL = DefaultUpdateLocationURL

    logwrite('[%s]--------------------------------------\n' % time.asctime())
    logwrite('sys.argv:%s \n' % (str(sys.argv)))
    logwrite('update repo: %s\n' % UpdateRepo)
    logwrite('update location: %s\n' % UpdateLocationURL)
    logwrite('is bin folder exist? : %s\n' % str(binDirExist))
    logwrite('executable filename is [%s]\n' % executable_filename)
    logwrite('shared location is     [%s]\n' % sharedStarterFilename)

    if (not binDirExist) or executable_filename != sharedStarterFilename:
        #if dhn is running - stop it.
        res = stop_all([    'dhnstarter.',
                            'dhnmain.',
                            'dhnview.',
                            'dhn.',
                            'datahaven', ])
        if res != 0:
            logwrite('can not stop dhn: %s\n' % res)
            sys.exit(res)

        logwrite('copy files to %s\n' % sharedPath('', 'bin'))
        try:
            copy_tree( os.path.dirname(executable_filename),
                       sharedPath('', 'bin'))
        except:
            logwrite('can not copy tree %s to %s\n' % (
                os.path.dirname(executable_filename),
                sharedPath('', 'bin')))

        logwrite('cd to bin\n')
        os.chdir(sharedPath('', 'bin'))

        try:
            os.remove(FilesDigestsLocalPath)
        except:
            logwrite('can not remove %s\n' % FilesDigestsLocalPath)

        exepath = sharedStarterFilename
        cmdargs = [StarterFilename,]
        if sys.argv.count('show') or sys.argv.count('install'):
            cmdargs.append('show')
        logwrite('launch %s cmdargs=%s' % (exepath, str(cmdargs)))
        os.spawnv(os.P_DETACH, exepath, cmdargs)
        sys.exit(0)

    else:
        os.chdir(sharedPath('', 'bin'))

    retcode = main()

    if retcode == 1:
        launch(sys.argv.count('show') or sys.argv.count('install'))

    elif retcode != 0:
        logwrite('retcode='+retcode+'\n')


if __name__ == "__main__":
    run()




